@@ -198,9 +198,11 @@ def test_geom_analysis_1d():
     analysis.get_datasets()
 
     assert len(analysis.datasets) == 1
-    assert len(analysis.datasets[0].aeff.energy.center) == 50
-    assert_allclose(analysis.datasets[0].aeff.energy.edges[0].to_value("TeV"), 0.03)
-    assert_allclose(analysis.datasets[0].aeff.energy.edges[-1].to_value("TeV"), 100)
+
+    axis = analysis.datasets[0].exposure.geom.axes["energy_true"]
+    assert axis.nbin == 50
+    assert_allclose(axis.edges[0].to_value("TeV"), 0.03)
+    assert_allclose(axis.edges[-1].to_value("TeV"), 100)
 
 
 @requires_data()
@@ -273,7 +275,7 @@ def test_analysis_ring_background():
     analysis.get_datasets()
     assert isinstance(analysis.datasets[0], MapDataset)
     assert_allclose(
-        analysis.datasets[0].background_model.map.data[0, 10, 10], 0.091552, rtol=1e-5
+        analysis.datasets[0].npred_background().data[0, 10, 10], 0.091799, rtol=1e-2
     )
 
 
@@ -312,7 +314,7 @@ def test_analysis_3d():
     analysis.get_observations()
     analysis.get_datasets()
     analysis.read_models(MODEL_FILE)
-    analysis.datasets["stacked"].background_model.tilt.frozen = False
+    analysis.datasets["stacked"].background_model.spectral_model.tilt.frozen = False
     analysis.run_fit()
     analysis.get_flux_points()
 
@@ -323,8 +325,8 @@ def test_analysis_3d():
     assert len(analysis.flux_points.data.table) == 2
     dnde = analysis.flux_points.data.table["dnde"].quantity
 
-    assert_allclose(dnde[0].value, 1.340073e-11, rtol=1e-2)
-    assert_allclose(dnde[-1].value, 3.483509e-13, rtol=1e-2)
+    assert_allclose(dnde[0].value, 1.376879e-11, rtol=1e-2)
+    assert_allclose(dnde[-1].value, 2.691466e-13, rtol=1e-2)
     assert_allclose(res["index"].value, 3.097613, rtol=1e-2)
     assert_allclose(res["tilt"].value, -0.207792, rtol=1e-2)
 
@@ -337,10 +339,17 @@ def test_analysis_3d_joint_datasets():
     analysis.get_observations()
     analysis.get_datasets()
     assert len(analysis.datasets) == 2
-    assert_allclose(analysis.datasets[0].background_model.norm.value, 1.031743694988066)
-    assert_allclose(analysis.datasets[0].background_model.tilt.value, 0.0)
+
     assert_allclose(
-        analysis.datasets[1].background_model.norm.value, 0.9776349021876344
+        analysis.datasets[0].background_model.spectral_model.norm.value,
+        1.031743694988066,
+    )
+    assert_allclose(
+        analysis.datasets[0].background_model.spectral_model.tilt.value, 0.0
+    )
+    assert_allclose(
+        analysis.datasets[1].background_model.spectral_model.norm.value,
+        0.9776349021876344,
     )
 
 
