@@ -21,13 +21,15 @@ def test_DMAnnihilation():
     channel = "b"
     massDM = 5 * u.TeV
     jfactor = 3.41e19 * u.Unit("GeV2 cm-5")
-    emin = 0.01 * u.TeV
-    emax = 10 * u.TeV
+    energy_min = 0.01 * u.TeV
+    energy_max = 10 * u.TeV
 
     model = DarkMatterAnnihilationSpectralModel(
         mass=massDM, channel=channel, jfactor=jfactor
     )
-    integral_flux = model.integral(emin=emin, emax=emax).to("cm-2 s-1")
+    integral_flux = model.integral(energy_min=energy_min, energy_max=energy_max).to(
+        "cm-2 s-1"
+    )
     differential_flux = model.evaluate(energy=1 * u.TeV, scale=1).to("cm-2 s-1 TeV-1")
 
     assert_quantity_allclose(integral_flux.value, 6.19575457e-14, rtol=1e-5)
