@@ -7,25 +7,20 @@ from astropy.coordinates.angle_utilities import angular_separation
 from astropy.table import Table
 from astropy.table import vstack as vstack_tables
 from astropy.units import Quantity, Unit
-from gammapy.maps import MapAxis, MapCoord, WcsNDMap
+from gammapy.maps import MapAxis, MapCoord, RegionGeom, WcsNDMap
 from gammapy.utils.fits import earth_location_from_dict
-from gammapy.utils.regions import make_region
 from gammapy.utils.scripts import make_path
 from gammapy.utils.testing import Checker
 from gammapy.utils.time import time_ref_from_dict
 
-__all__ = ["EventListBase", "EventList", "EventListLAT"]
+__all__ = ["EventList"]
 
 log = logging.getLogger(__name__)
 
 
-class EventListBase:
+class EventList:
     """Event list.
 
-    This class represents the base for two different event lists:
-    - EventList: targeted for IACT event lists
-    - EventListLAT: targeted for Fermi-LAT event lists
-
     Event list data is stored in ``table`` (`~astropy.table.Table`) data member.
 
     The most important reconstructed event parameters
@@ -79,7 +74,7 @@ class EventListBase:
         return cls(table=table)
 
     @classmethod
-    def stack(cls, event_lists, **kwargs):
+    def from_stack(cls, event_lists, **kwargs):
         """Stack (concatenate) list of event lists.
 
         Calls `~astropy.table.vstack`.
@@ -93,26 +88,47 @@ class EventListBase:
         stacked_table = vstack_tables(tables, **kwargs)
         return cls(stacked_table)
 
+    def stack(self, other):
+        """Stack with another EventList in place.
+
+        Calls `~astropy.table.vstack`.
+
+        Parameters
+        ----------
+        other : `~gammapy.data.EventList`
+            Event list to stack to self
+        """
+        self.table = vstack_tables([self.table, other.table])
+
     def __str__(self):
-        ss = (
-            "EventList info:\n"
-            f"- Number of events: {len(self.table)}\n"
-            f"- Median energy: {np.median(self.energy.value):.3g} {self.energy.unit}\n"
-        )
+        info = self.__class__.__name__ + "\n"
+        info += "-" * len(self.__class__.__name__) + "\n\n"
 
-        if "OBS_ID" in self.table.meta:
-            ss += "- OBS_ID = {}".format(self.table.meta["OBS_ID"])
+        instrument = self.table.meta.get("INSTRUME")
+        info += f"\tInstrument       : {instrument}\n"
 
-        # TODO: add time, RA, DEC and if present GLON, GLAT info ...
+        telescope = self.table.meta.get("TELESCOP")
+        info += f"\tTelescope        : {telescope}\n"
 
-        if "AZ" in self.table.colnames:
-            # TODO: azimuth should be circular median
-            ss += "- Median azimuth: {}\n".format(np.median(self.table["AZ"]))
+        obs_id = self.table.meta.get("OBS_ID", "")
+        info += f"\tObs. ID          : {obs_id}\n\n"
 
-        if "ALT" in self.table.colnames:
-            ss += "- Median altitude: {}\n".format(np.median(self.table["ALT"]))
+        info += f"\tNumber of events : {len(self.table)}\n"
 
-        return ss
+        rate = len(self.table) / self.observation_time_duration
+        info += f"\tEvent rate       : {rate:.3f}\n\n"
+
+        info += f"\tTime start       : {self.observation_time_start}\n"
+        info += f"\tTime stop        : {self.observation_time_stop}\n\n"
+
+        info += f"\tMin. energy      : {np.min(self.energy):.2e}\n"
+        info += f"\tMax. energy      : {np.max(self.energy):.2e}\n"
+        info += f"\tMedian energy    : {np.median(self.energy):.2e}\n\n"
+
+        if self.is_pointed_observation:
+            offset_max = np.max(self.offset)
+            info += f"\tMax. offset      : {offset_max:.1f}\n"
+        return info.expandtabs(tabsize=2)
 
     @property
     def time_ref(self):
@@ -138,7 +154,7 @@ class EventListBase:
         return self.time_ref + Quantity(self.table.meta["TSTART"], "second")
 
     @property
-    def observation_time_end(self):
+    def observation_time_stop(self):
         """Observation stop time (`~astropy.time.Time`)."""
         return self.time_ref + Quantity(self.table.meta["TSTOP"], "second")
 
@@ -161,6 +177,14 @@ class EventListBase:
         """Event energies (`~astropy.units.Quantity`)."""
         return self.table["ENERGY"].quantity
 
+    @property
+    def galactic_median(self):
+        """Median position in radec"""
+        galactic = self.galactic
+        median_lon = np.median(galactic.l.wrap_at("180d"))
+        median_lat = np.median(galactic.b)
+        return SkyCoord(median_lon, median_lat, frame="galactic")
+
     def select_row_subset(self, row_specifier):
         """Select table row subset.
 
@@ -190,13 +214,13 @@ class EventListBase:
         table = self.table[row_specifier]
         return self.__class__(table=table)
 
-    def select_energy(self, energy_band):
+    def select_energy(self, energy_range):
         """Select events in energy band.
 
         Parameters
         ----------
-        energy_band : `~astropy.units.Quantity`
-            Energy band ``[energy_min, energy_max)``
+        energy_range : `~astropy.units.Quantity`
+            Energy range ``[energy_min, energy_max)``
 
         Returns
         -------
@@ -208,12 +232,12 @@ class EventListBase:
         >>> from astropy.units import Quantity
         >>> from gammapy.data import EventList
         >>> event_list = EventList.read('events.fits')
-        >>> energy_band = Quantity([1, 20], 'TeV')
+        >>> energy_range = Quantity([1, 20], 'TeV')
         >>> event_list = event_list.select_energy()
         """
         energy = self.energy
-        mask = energy_band[0] <= energy
-        mask &= energy < energy_band[1]
+        mask = energy_range[0] <= energy
+        mask &= energy < energy_range[1]
         return self.select_row_subset(mask)
 
     def select_time(self, time_interval):
@@ -249,8 +273,8 @@ class EventListBase:
         event_list : `EventList`
             Copy of event list with selection applied.
         """
-        region = make_region(region)
-        mask = region.contains(self.radec, wcs)
+        geom = RegionGeom.create(region, wcs=wcs)
+        mask = geom.contains(self.radec)
         return self.select_row_subset(mask)
 
     def select_parameter(self, parameter, band):
@@ -280,11 +304,11 @@ class EventListBase:
         mask &= self.table[parameter].quantity < band[1]
         return self.select_row_subset(mask)
 
-    def _default_plot_ebounds(self):
+    def _default_plot_energy_edges(self):
         energy = self.energy
         return MapAxis.from_energy_bounds(energy.min(), energy.max(), 50).edges
 
-    def plot_energy(self, ax=None, ebounds=None, **kwargs):
+    def plot_energy(self, ax=None, energy_edges=None, **kwargs):
         """Plot counts as a function of energy."""
         import matplotlib.pyplot as plt
 
@@ -293,12 +317,12 @@ class EventListBase:
         kwargs.setdefault("log", True)
         kwargs.setdefault("histtype", "step")
 
-        if ebounds is None:
-            ebounds = self._default_plot_ebounds()
+        if energy_edges is None:
+            energy_edges = self._default_plot_energy_edges()
 
-        unit = ebounds.unit
+        unit = energy_edges.unit
 
-        ax.hist(self.energy.to_value(unit), bins=ebounds.value, **kwargs)
+        ax.hist(self.energy.to_value(unit), bins=energy_edges.value, **kwargs)
         ax.loglog()
         ax.set_xlabel(f"Energy ({unit})")
         ax.set_ylabel("Counts")
@@ -327,8 +351,8 @@ class EventListBase:
 
         ax.set_xlabel("Time (sec)")
         ax.set_ylabel("Counts")
-        y, x_edges = np.histogram(time, bins=30)
-        # x = (x_edges[1:] + x_edges[:-1]) / 2
+        y, x_edges = np.histogram(time, bins=20)
+
         xerr = np.diff(x_edges) / 2
         x = x_edges[:-1] + xerr
         yerr = np.sqrt(y)
@@ -398,37 +422,42 @@ class EventListBase:
         ax = plt.gca() if ax is None else ax
 
         if center is None:
-            center = self.pointing_radec
+            center = self._plot_center
 
         offset2 = center.separation(self.radec).deg ** 2
 
+        kwargs.setdefault("histtype", "step")
+        kwargs.setdefault("bins", 30)
+
         ax.hist(offset2, **kwargs)
         ax.set_xlabel("Offset^2 (deg^2)")
         ax.set_ylabel("Counts")
 
         return ax
 
-    def plot_energy_offset(self, ax=None):
+    def plot_energy_offset(self, ax=None, center=None):
         """Plot counts histogram with energy and offset axes."""
         import matplotlib.pyplot as plt
         from matplotlib.colors import LogNorm
 
         ax = plt.gca() if ax is None else ax
 
-        energy_bounds = self._default_plot_ebounds().to_value("TeV")
-        offsetangle = self.pointing_radec.separation(self.radec).max().to("deg").value
-        offset_bounds = np.linspace(0, offsetangle, 30)
+        if center is None:
+            center = self._plot_center
+
+        energy_bounds = self._default_plot_energy_edges().to_value(self.energy.unit)
+        offset = center.separation(self.radec)
+        offset_max = offset.max()
+        offset_bounds = np.linspace(0, offset_max.deg, 30)
 
         counts = np.histogram2d(
-            x=self.energy.value,
-            y=self.offset.value,
-            bins=(energy_bounds, offset_bounds),
+            x=self.energy.value, y=offset.deg, bins=(energy_bounds, offset_bounds),
         )[0]
 
         ax.pcolormesh(energy_bounds, offset_bounds, counts.T, norm=LogNorm())
         ax.set_xscale("log")
         ax.set_xlabel(f"Energy ({self.energy.unit})")
-        ax.set_ylabel(f"Offset ({self.offset.unit})")
+        ax.set_ylabel(f"Offset ({offset.unit})")
 
     def check(self, checks="all"):
         """Run checks.
@@ -477,28 +506,6 @@ class EventListBase:
         valid = values > 0
         return self.select_row_subset(valid)
 
-
-class EventList(EventListBase):
-    """Event list for IACT dataset.
-
-    Data format specification: :ref:`gadf:iact-events`
-
-    For further information, see the base class: `~gammapy.data.EventListBase`.
-
-    Parameters
-    ----------
-    table : `~astropy.table.Table`
-        Event list table
-
-    Examples
-    --------
-    To load an example H.E.S.S. event list:
-
-    >>> from gammapy.data import EventList
-    >>> filename = '$GAMMAPY_DATA/hess-dl3-dr1/data/hess_dl3_dr1_obs_id_023523.fits.gz'
-    >>> events = EventList.read(filename)
-    """
-
     @property
     def observatory_earth_location(self):
         """Observatory location (`~astropy.coordinates.EarthLocation`)."""
@@ -511,7 +518,8 @@ class EventList(EventListBase):
         This is a keyword related to IACTs
         The wall time, including dead-time.
         """
-        return Quantity(self.table.meta["ONTIME"], "second")
+        time_delta = (self.observation_time_stop - self.observation_time_start).sec
+        return Quantity(time_delta, "s")
 
     @property
     def observation_live_time_duration(self):
@@ -575,6 +583,14 @@ class EventList(EventListBase):
         offset = center.separation(position)
         return Angle(offset, unit="deg")
 
+    @property
+    def offset_from_median(self):
+        """Event offset from the median position (`~astropy.coordinates.Angle`)."""
+        position = self.radec
+        center = self.galactic_median
+        offset = center.separation(position)
+        return Angle(offset, unit="deg")
+
     def select_offset(self, offset_band):
         """Select events in offset band.
 
@@ -593,84 +609,94 @@ class EventList(EventListBase):
         mask &= offset < offset_band[1]
         return self.select_row_subset(mask)
 
-    def peek(self):
-        """Quick look plots."""
+    @property
+    def is_pointed_observation(self):
+        """Whether observation is pointed"""
+        return "RA_PNT" in self.table.meta
+
+    def peek(self, allsky=False):
+        """Quick look plots.
+
+        Parameters
+        ----------
+        allsky : bool
+            Wheter to look at the events allsky
+        """
         import matplotlib.pyplot as plt
+        import matplotlib.gridspec as gridspec
 
-        fig, axes = plt.subplots(nrows=2, ncols=3, figsize=(12, 8))
-        self.plot_energy(ax=axes[0, 0])
+        if allsky:
+            gs = gridspec.GridSpec(nrows=2, ncols=2)
+            fig = plt.figure(figsize=(8, 8))
+        else:
+            gs = gridspec.GridSpec(nrows=2, ncols=3)
+            fig = plt.figure(figsize=(12, 8))
 
-        off_max = self.pointing_radec.separation(self.radec).max().deg
-        bins = np.linspace(start=0, stop=off_max ** 2, num=30)
-        self.plot_offset2_distribution(ax=axes[0, 1], bins=bins)
+        # energy plot
+        ax_energy = fig.add_subplot(gs[1, 0])
+        self.plot_energy(ax=ax_energy)
 
-        self.plot_time(ax=axes[0, 2])
+        # offset plots
+        if not allsky:
+            ax_offset = fig.add_subplot(gs[0, 1])
+            self.plot_offset2_distribution(ax=ax_offset)
+            ax_energy_offset = fig.add_subplot(gs[0, 2])
+            self.plot_energy_offset(ax=ax_energy_offset)
 
-        axes[1, 0].axis("off")
-        m = self._counts_image()
-        ax = plt.subplot(2, 3, 4, projection=m.geom.wcs)
-        m.plot(ax=ax, stretch="linear")
+        # time plot
+        ax_time = fig.add_subplot(gs[1, 1])
+        self.plot_time(ax=ax_time)
 
-        self.plot_energy_offset(ax=axes[1, 1])
+        # image plot
+        m = self._counts_image(allsky=allsky)
+        if allsky:
+            ax_image = fig.add_subplot(gs[0, :], projection=m.geom.wcs)
+        else:
+            ax_image = fig.add_subplot(gs[0, 0], projection=m.geom.wcs)
+        m.plot(ax=ax_image, stretch="sqrt", vmin=0)
+        plt.subplots_adjust(wspace=0.3)
 
-        self._plot_text_summary(ax=axes[1, 2])
-        plt.subplots_adjust(hspace=0.3, wspace=0.2)
+    @property
+    def _plot_center(self):
+        if self.is_pointed_observation:
+            return self.pointing_radec
+        else:
+            return self.galactic_median
 
-    def _plot_text_summary(self, ax):
-        ax.axis("off")
-        txt = str(self)
-        ax.text(0, 1, txt, fontsize=12, verticalalignment="top")
+    @property
+    def _plot_width(self):
+        if self.is_pointed_observation:
+            offset = self.offset
+        else:
+            offset = self.offset_from_median
+
+        return 2 * offset.max()
+
+    def _counts_image(self, allsky):
+        if allsky:
+            opts = {
+                "npix": (360, 180),
+                "binsz": 1.0,
+                "proj": "AIT",
+                "frame": "galactic",
+            }
+        else:
+            opts = {
+                "width": self._plot_width,
+                "binsz": 0.05,
+                "proj": "TAN",
+                "frame": "galactic",
+                "skydir": self._plot_center,
+            }
 
-    def _counts_image(self):
-        width = self.galactic.b.max().deg - self.galactic.b.min().deg
-        opts = {
-            "width": (width, width),
-            "binsz": 0.05,
-            "proj": "TAN",
-            "frame": "galactic",
-            "skydir": self.pointing_radec,
-        }
         m = WcsNDMap.create(**opts)
         m.fill_by_coord(self.radec)
         m = m.smooth(width=0.5)
         return m
 
-    def plot_image(self):
+    def plot_image(self, allsky=False):
         """Quick look counts map sky plot."""
-        m = self._counts_image()
-        m.plot(stretch="sqrt")
-
-
-class EventListLAT(EventListBase):
-    """Event list for Fermi-LAT dataset.
-
-    Fermi-LAT data products
-    https://fermi.gsfc.nasa.gov/ssc/data/analysis/documentation/Cicerone/Cicerone_Data/LAT_DP.html
-    Data format specification (columns)
-    https://fermi.gsfc.nasa.gov/ssc/data/analysis/documentation/Cicerone/Cicerone_Data/LAT_Data_Columns.html
-
-    For further information, see the base class: `~gammapy.data.EventListBase`.
-
-    Parameters
-    ----------
-    table : `~astropy.table.Table`
-        Event list table
-
-    Examples
-    --------
-    To load an example Fermi-LAT event list:
-
-    >>> from gammapy.data import EventListLAT
-    >>> filename = "$GAMMAPY_DATA/fermi-3fhl-gc/fermi-3fhl-gc-events.fits.gz"
-    >>> events = EventListLAT.read(filename)
-    """
-
-    def plot_image(self):
-        """Quick look counts map sky plot."""
-        from gammapy.maps import WcsNDMap
-
-        m = WcsNDMap.create(npix=(360, 180), binsz=1.0, proj="AIT", frame="galactic")
-        m.fill_by_coord(self.radec)
+        m = self._counts_image(allsky=allsky)
         m.plot(stretch="sqrt")
 
 
@@ -774,7 +800,7 @@ class EventListChecker(Checker):
         if dt.min() < self.accuracy["time"].to_value("s"):
             yield self._record(level="error", msg="Event times before obs start time")
 
-        dt = (self.event_list.time - self.event_list.observation_time_end).sec
+        dt = (self.event_list.time - self.event_list.observation_time_stop).sec
         if dt.max() > self.accuracy["time"].to_value("s"):
             yield self._record(level="error", msg="Event times after the obs end time")
 
