@@ -2,12 +2,18 @@
 import abc
 import collections.abc
 import copy
+import logging
 import numpy as np
-from gammapy.maps import Map
-from gammapy.modeling.models import Models
+from astropy import units as u
+from astropy.table import Table, vstack
+from gammapy.data import GTI
+from gammapy.modeling.models import DatasetModels, Models
 from gammapy.utils.scripts import make_name, make_path, read_yaml, write_yaml
 from gammapy.utils.table import table_from_row_data
 
+log = logging.getLogger(__name__)
+
+
 __all__ = ["Dataset", "Datasets"]
 
 
@@ -18,9 +24,9 @@ class Dataset(abc.ABC):
 
     For now, see existing examples in Gammapy how this works:
 
-    - `gammapy.cube.MapDataset`
-    - `gammapy.spectrum.SpectrumDataset`
-    - `gammapy.spectrum.FluxPointsDataset`
+    - `gammapy.datasets.MapDataset`
+    - `gammapy.datasets.SpectrumDataset`
+    - `gammapy.datasets.FluxPointsDataset`
     """
 
     _residuals_labels = {
@@ -29,31 +35,27 @@ class Dataset(abc.ABC):
         "diff/sqrt(model)": "(data - model) / sqrt(model)",
     }
 
+    @property
+    @abc.abstractmethod
+    def tag(self):
+        pass
+
     @property
     def mask(self):
         """Combined fit and safe mask"""
-        mask_safe = (
-            self.mask_safe.data if isinstance(self.mask_safe, Map) else self.mask_safe
-        )
-        mask_fit = (
-            self.mask_fit.data if isinstance(self.mask_fit, Map) else self.mask_fit
-        )
-        if mask_safe is not None and mask_fit is not None:
-            mask = mask_safe & mask_fit
-        elif mask_fit is not None:
-            mask = mask_fit
-        elif mask_safe is not None:
-            mask = mask_safe
-        else:
-            mask = None
-        return mask
+        if self.mask_safe is not None and self.mask_fit is not None:
+            return self.mask_safe & self.mask_fit
+        elif self.mask_fit is not None:
+            return self.mask_fit
+        elif self.mask_safe is not None:
+            return self.mask_safe
 
     def stat_sum(self):
         """Total statistic given the current model parameters."""
         stat = self.stat_array()
 
         if self.mask is not None:
-            stat = stat[self.mask]
+            stat = stat[self.mask.data]
 
         return np.sum(stat, dtype=np.float64)
 
@@ -64,7 +66,16 @@ class Dataset(abc.ABC):
     def copy(self, name=None):
         """A deep copy."""
         new = copy.deepcopy(self)
-        new._name = make_name(name)
+        name = make_name(name)
+        new._name = name
+
+        # propagate new dataset name
+        if new._models is not None:
+            for m in new._models:
+                if m.datasets_names is not None:
+                    for k, d in enumerate(m.datasets_names):
+                        if d == self.name:
+                            m.datasets_names[k] = name
         return new
 
     @staticmethod
@@ -99,11 +110,9 @@ class Datasets(collections.abc.MutableSequence):
 
         if isinstance(datasets, Datasets):
             datasets = datasets._datasets
-        elif isinstance(datasets, list):
-            datasets = datasets
         elif isinstance(datasets, Dataset):
             datasets = [datasets]
-        else:
+        elif not isinstance(datasets, list):
             raise TypeError(f"Invalid type: {datasets!r}")
 
         unique_names = []
@@ -130,17 +139,24 @@ class Datasets(collections.abc.MutableSequence):
         Duplicate model objects have been removed.
         The order of the unique models remains.
         """
-        unique_models = []
-        for d in self._datasets:
-            if d.models is not None:
-                for model in d.models:
-                    if model not in unique_models:
-                        if (
-                            model.datasets_names is None
-                            or d.name in model.datasets_names
-                        ):
-                            unique_models.append(model)
-        return Models(unique_models)
+        models = {}
+
+        for dataset in self:
+            if dataset.models is not None:
+                for model in dataset.models:
+                    models[model] = model
+
+        return DatasetModels(list(models.keys()))
+
+    @models.setter
+    def models(self, models):
+        """Unique models (`~gammapy.modeling.Models`).
+
+        Duplicate model objects have been removed.
+        The order of the unique models remains.
+        """
+        for dataset in self:
+            dataset.models = models
 
     @property
     def names(self):
@@ -161,6 +177,12 @@ class Datasets(collections.abc.MutableSequence):
         """Whether all contained datasets have the same data shape"""
         return len(set(_.data_shape[0] for _ in self)) == 1
 
+    @property
+    def energy_axes_are_aligned(self):
+        """Whether all contained datasets have aligned energy axis"""
+        axes = [d.counts.geom.axes["energy"] for d in self]
+        return np.all([axes[0].is_aligned(ax) for ax in axes])
+
     def stat_sum(self):
         """Compute joint likelihood"""
         stat_sum = 0
@@ -169,69 +191,189 @@ class Datasets(collections.abc.MutableSequence):
             stat_sum += dataset.stat_sum()
         return stat_sum
 
+    def select_time(self, t_min, t_max, atol="1e-6 s"):
+        """Select datasets in a given time interval.
+
+        Parameters
+        ----------
+        t_min, t_max : `~astropy.time.Time`
+            Time interval
+        atol : `~astropy.units.Quantity`
+            Tolerance value for time comparison with different scale. Default 1e-6 sec.
+
+        Returns
+        -------
+        datasets : `Datasets`
+            Datasets in the given time interval.
+
+        """
+        atol = u.Quantity(atol)
+
+        datasets = []
+
+        for dataset in self:
+            t_start = dataset.gti.time_start[0]
+            t_stop = dataset.gti.time_stop[-1]
+
+            if t_start >= (t_min - atol) and t_stop <= (t_max + atol):
+                datasets.append(dataset)
+
+        return self.__class__(datasets)
+
+    def slice_by_energy(self, energy_min, energy_max):
+        """Select and slice datasets in energy range
+
+        Parameters
+        ----------
+        energy_min, energy_max : `~astropy.units.Quantity`
+            Energy bounds to compute the flux point for.
+
+        Returns
+        -------
+        datasets : Datasets
+            Datasets
+
+        """
+        datasets = []
+
+        for dataset in self:
+            try:
+                dataset_sliced = dataset.slice_by_energy(
+                    energy_min=energy_min, energy_max=energy_max, name=dataset.name,
+                )
+            except ValueError:
+                log.info(
+                    f"Dataset {dataset.name} does not contribute in the energy range"
+                )
+                continue
+
+            datasets.append(dataset_sliced)
+
+        return self.__class__(datasets=datasets)
+
+    @property
+    # TODO: make this a method to support different methods?
+    def energy_ranges(self):
+        """Get global energy range of datasets.
+
+        The energy range is derived as the minimum / maximum of the energy
+        ranges of all datasets.
+
+        Returns
+        -------
+        energy_min, energy_max : `~astropy.units.Quantity`
+            Energy range.
+        """
+
+        energy_mins, energy_maxs = [], []
+
+        for dataset in self:
+            energy_axis = dataset.counts.geom.axes["energy"]
+            energy_mins.append(energy_axis.edges[0])
+            energy_maxs.append(energy_axis.edges[-1])
+
+        return u.Quantity(energy_mins), u.Quantity(energy_maxs)
+
     def __str__(self):
         str_ = self.__class__.__name__ + "\n"
-        str_ += "--------\n"
+        str_ += "--------\n\n"
 
         for idx, dataset in enumerate(self):
-            str_ += f"idx={idx}, id={hex(id(dataset))!r}, name={dataset.name!r}\n"
+            str_ += f"Dataset {idx}: \n\n"
+            str_ += f"\tType       : {dataset.tag}\n"
+            str_ += f"\tName       : {dataset.name}\n"
+            try:
+                instrument = set(dataset.meta_table["TELESCOP"]).pop()
+            except (KeyError, TypeError):
+                instrument = ""
+            str_ += f"\tInstrument : {instrument}\n"
+            if dataset.models:
+                names = dataset.models.names
+            else:
+                names = ""
+            str_ += f"\tModels     : {names}\n\n"
 
-        return str_
+        return str_.expandtabs(tabsize=2)
 
     def copy(self):
         """A deep copy."""
         return copy.deepcopy(self)
 
     @classmethod
-    def read(cls, filedata, filemodel):
+    def read(cls, filename, filename_models=None, lazy=True, cache=True):
         """De-serialize datasets from YAML and FITS files.
 
         Parameters
         ----------
-        filedata : str
-            filepath to yaml datasets file
-        filemodel : str
-            filepath to yaml models file
+        filename : str or `Path`
+            File path or name of datasets yaml file
+        filename_models : str or `Path`
+            File path or name of models fyaml ile
+        lazy : bool
+            Whether to lazy load data into memory
+        cache : bool
+            Whether to cache the data after loading.
 
         Returns
         -------
-        dataset : 'gammapy.modeling.Datasets'
+        dataset : `gammapy.datasets.Datasets`
             Datasets
         """
-        from . import DATASETS
+        from . import DATASET_REGISTRY
 
-        models = Models.read(make_path(filemodel))
-        data_list = read_yaml(make_path(filedata))
+        filename = make_path(filename)
+        data_list = read_yaml(filename)
 
         datasets = []
         for data in data_list["datasets"]:
-            dataset = DATASETS.get_cls(data["type"]).from_dict(data, models)
-            datasets.append(dataset)
-        return cls(datasets)
+            path = filename.parent
 
-    def write(self, path, prefix="", overwrite=False):
+            if (path / data["filename"]).exists():
+                data["filename"] = str(make_path(path / data["filename"]))
+
+            dataset_cls = DATASET_REGISTRY.get_cls(data["type"])
+            dataset = dataset_cls.from_dict(data, lazy=lazy, cache=cache)
+            datasets.append(dataset)
+
+        datasets = cls(datasets)
+
+        if filename_models:
+            datasets.models = Models.read(filename_models)
+
+        return datasets
+
+    def write(
+        self, filename, filename_models=None, overwrite=False, write_covariance=True
+    ):
         """Serialize datasets to YAML and FITS files.
 
         Parameters
         ----------
-        path : `pathlib.Path`
-            path to write files
-        prefix : str
-            common prefix of file names
+        filename : str or `Path`
+            File path or name of datasets yaml file
+        filename_models : str or `Path`
+            File path or name of models fyaml ile
         overwrite : bool
             overwrite datasets FITS files
+        write_covariance : bool
+            save covariance or not
         """
+        path = make_path(filename).resolve()
+
+        data = {"datasets": []}
 
-        path = make_path(path)
-        datasets_dictlist = []
         for dataset in self._datasets:
-            filename = path / f"{prefix}_data_{dataset.name}.fits"
-            dataset.write(filename, overwrite)
-            datasets_dictlist.append(dataset.to_dict(filename=filename))
-        datasets_dict = {"datasets": datasets_dictlist}
+            name = dataset.name.replace(" ", "_")
+            filename = f"{name}.fits"
+            dataset.write(path.parent / filename, overwrite=overwrite)
+            data["datasets"].append(dataset.to_dict(filename=filename))
 
-        write_yaml(datasets_dict, path / f"{prefix}_datasets.yaml", sort_keys=False)
-        self.models.write(path / f"{prefix}_models.yaml", overwrite=overwrite)
+        write_yaml(data, path, sort_keys=False)
+
+        if filename_models:
+            self.models.write(
+                filename_models, overwrite=overwrite, write_covariance=write_covariance,
+            )
 
     def stack_reduce(self, name=None):
         """Reduce the Datasets to a unique Dataset by stacking them together.
@@ -254,7 +396,7 @@ class Datasets(collections.abc.MutableSequence):
             dataset.stack(ds)
         return dataset
 
-    def info_table(self, cumulative=False):
+    def info_table(self, cumulative=False, region=None):
         """Get info table for datasets.
 
         Parameters
@@ -270,7 +412,7 @@ class Datasets(collections.abc.MutableSequence):
         if not self.is_all_same_type:
             raise ValueError("Info table not supported for mixed dataset type.")
 
-        stacked = self[0].copy(name="stacked")
+        stacked = self[0].copy(name=self[0].name)
 
         rows = [stacked.info_dict()]
 
@@ -285,6 +427,33 @@ class Datasets(collections.abc.MutableSequence):
 
         return table_from_row_data(rows=rows)
 
+    # TODO: merge with meta table?
+    @property
+    def gti(self):
+        """GTI table"""
+        time_intervals = []
+
+        for dataset in self:
+            if dataset.gti is not None:
+                interval = (dataset.gti.time_start[0], dataset.gti.time_stop[-1])
+                time_intervals.append(interval)
+
+        return GTI.from_time_intervals(time_intervals)
+
+    @property
+    def meta_table(self):
+        """Meta table"""
+        tables = [d.meta_table for d in self]
+
+        if np.all([table is None for table in tables]):
+            meta_table = Table()
+        else:
+            meta_table = vstack(tables)
+
+        meta_table.add_column([d.tag for d in self], index=0, name="TYPE")
+        meta_table.add_column(self.names, index=0, name="NAME")
+        return meta_table
+
     def __getitem__(self, key):
         return self._datasets[self.index(key)]
 
