@@ -4,14 +4,14 @@ import numpy as np
 from astropy import units as u
 from astropy.io.registry import IORegistryError
 from astropy.table import Table, vstack
-from gammapy.datasets import MapDataset
+from gammapy.datasets import Datasets
 from gammapy.modeling.models import PowerLawSpectralModel
 from gammapy.utils.interpolation import interpolate_profile
 from gammapy.utils.scripts import make_path
 from gammapy.utils.table import table_from_row_data, table_standardise_units_copy
+from .core import Estimator
 from .flux import FluxEstimator
 
-
 __all__ = ["FluxPoints", "FluxPointsEstimator"]
 
 log = logging.getLogger(__name__)
@@ -22,15 +22,7 @@ REQUIRED_COLUMNS = {
     "flux": ["e_min", "e_max", "flux"],
     "eflux": ["e_min", "e_max", "eflux"],
     # TODO: extend required columns
-    "likelihood": [
-        "e_min",
-        "e_max",
-        "e_ref",
-        "ref_dnde",
-        "norm",
-        "norm_scan",
-        "stat_scan",
-    ],
+    "likelihood": ["e_min", "e_max", "e_ref", "ref_dnde", "norm",],
 }
 
 OPTIONAL_COLUMNS = {
@@ -38,6 +30,7 @@ OPTIONAL_COLUMNS = {
     "e2dnde": ["e2dnde_err", "e2dnde_errp", "e2dnde_errn", "e2dnde_ul", "is_ul"],
     "flux": ["flux_err", "flux_errp", "flux_errn", "flux_ul", "is_ul"],
     "eflux": ["eflux_err", "eflux_errp", "eflux_errn", "eflux_ul", "is_ul"],
+    "likelihood": ["norm_scan", "stat_scan"],
 }
 
 DEFAULT_UNIT = {
@@ -75,7 +68,7 @@ class FluxPoints:
     The `FluxPoints` object is most easily created by reading a file with
     flux points given in one of the formats documented above::
 
-        from gammapy.spectrum import FluxPoints
+        from gammapy.estimators import FluxPoints
         filename = '$GAMMAPY_DATA/hawc_crab/HAWC19_flux_points.fits'
         flux_points = FluxPoints.read(filename)
         flux_points.plot()
@@ -87,7 +80,7 @@ class FluxPoints:
 
         from astropy import units as u
         from astropy.table import Table
-        from gammapy.spectrum import FluxPoints
+        from gammapy.estimators import FluxPoints
         from gammapy.modeling.models import PowerLawSpectralModel
 
         table = Table()
@@ -106,7 +99,7 @@ class FluxPoints:
 
         from astropy import units as u
         from astropy.table import Table
-        from gammapy.spectrum import FluxPoints
+        from gammapy.estimators import FluxPoints
 
         table = Table.read('$GAMMAPY_DATA/tests/spectrum/flux_points/flux_points_ctb_37b.txt',
                            format='ascii.csv', delimiter=' ', comment='#')
@@ -254,7 +247,7 @@ class FluxPoints:
 
         Examples
         --------
-        >>> from gammapy.spectrum import FluxPoints
+        >>> from gammapy.estimators import FluxPoints
         >>> filename = '$GAMMAPY_DATA/tests/spectrum/flux_points/flux_points.fits'
         >>> flux_points = FluxPoints.read(filename)
         >>> print(flux_points)
@@ -269,17 +262,19 @@ class FluxPoints:
         table_drop_ul = self.table[~self.is_ul]
         return self.__class__(table_drop_ul)
 
-    def _flux_to_dnde(self, e_ref, table, model, pwl_approx):
+    def _flux_to_dnde(self, energy_ref, table, model, pwl_approx):
         if model is None:
             model = PowerLawSpectralModel()
 
-        e_min, e_max = self.e_min, self.e_max
+        energy_min, energy_max = self.energy_min, self.energy_max
 
         flux = table["flux"].quantity
-        dnde = self._dnde_from_flux(flux, model, e_ref, e_min, e_max, pwl_approx)
+        dnde = self._dnde_from_flux(
+            flux, model, energy_ref, energy_min, energy_max, pwl_approx
+        )
 
         # Add to result table
-        table["e_ref"] = e_ref
+        table["e_ref"] = energy_ref
         table["dnde"] = dnde
 
         if "flux_err" in table.colnames:
@@ -292,18 +287,18 @@ class FluxPoints:
         if "flux_ul" in table.colnames:
             flux_ul = table["flux_ul"].quantity
             dnde_ul = self._dnde_from_flux(
-                flux_ul, model, e_ref, e_min, e_max, pwl_approx
+                flux_ul, model, energy_ref, energy_min, energy_max, pwl_approx
             )
             table["dnde_ul"] = dnde_ul
 
         return table
 
     @staticmethod
-    def _dnde_to_e2dnde(e_ref, table):
+    def _dnde_to_e2dnde(energy_ref, table):
         for suffix in ["", "_ul", "_err", "_errp", "_errn"]:
             try:
                 data = table["dnde" + suffix].quantity
-                table["e2dnde" + suffix] = (e_ref ** 2 * data).to(
+                table["e2dnde" + suffix] = (energy_ref ** 2 * data).to(
                     DEFAULT_UNIT["e2dnde"]
                 )
             except KeyError:
@@ -312,11 +307,13 @@ class FluxPoints:
         return table
 
     @staticmethod
-    def _e2dnde_to_dnde(e_ref, table):
+    def _e2dnde_to_dnde(energy_ref, table):
         for suffix in ["", "_ul", "_err", "_errp", "_errn"]:
             try:
                 data = table["e2dnde" + suffix].quantity
-                table["dnde" + suffix] = (data / e_ref ** 2).to(DEFAULT_UNIT["dnde"])
+                table["dnde" + suffix] = (data / energy_ref ** 2).to(
+                    DEFAULT_UNIT["dnde"]
+                )
             except KeyError:
                 continue
 
@@ -337,13 +334,13 @@ class FluxPoints:
             does not matter. Still it is recommended to use something with the right
             scale and units. E.g. `amplitude = 1e-12 * u.Unit('cm-2 s-1 TeV-1')`
         method : {'lafferty', 'log_center', 'table'}
-            Flux points `e_ref` estimation method:
+            Flux points `energy_ref` estimation method:
 
-                * `'laferty'` Lafferty & Wyatt model-based e_ref
-                * `'log_center'` log bin center e_ref
-                * `'table'` using column 'e_ref' from input flux_points
+                * `'laferty'` Lafferty & Wyatt model-based energy_ref
+                * `'log_center'` log bin center energy_ref
+                * `'table'` using column 'energy_ref' from input flux_points
         pwl_approx : bool
-            Use local power law appoximation at e_ref to compute differential flux
+            Use local power law appoximation at energy_ref to compute differential flux
             from the integral flux. This method is used by the Fermi-LAT catalogs.
 
         Returns
@@ -354,7 +351,7 @@ class FluxPoints:
 
         Examples
         --------
-        >>> from gammapy.spectrum import FluxPoints
+        >>> from gammapy.estimators import FluxPoints
         >>> from gammapy.modeling.models import PowerLawSpectralModel
         >>> filename = '$GAMMAPY_DATA/tests/spectrum/flux_points/flux_points.fits'
         >>> flux_points = FluxPoints.read(filename)
@@ -369,23 +366,25 @@ class FluxPoints:
         table = self.table.copy()
 
         if self.sed_type == "flux" and sed_type == "dnde":
-            # Compute e_ref
+            # Compute energy_ref
             if method == "table":
-                e_ref = table["e_ref"].quantity
+                energy_ref = table["e_ref"].quantity
             elif method == "log_center":
-                e_ref = np.sqrt(self.e_min * self.e_max)
+                energy_ref = np.sqrt(self.energy_min * self.energy_max)
             elif method == "lafferty":
-                # set e_ref that it represents the mean dnde in the given energy bin
-                e_ref = self._e_ref_lafferty(model, self.e_min, self.e_max)
+                # set energy_ref that it represents the mean dnde in the given energy bin
+                energy_ref = self._energy_ref_lafferty(
+                    model, self.energy_min, self.energy_max
+                )
             else:
                 raise ValueError(f"Invalid method: {method}")
-            table = self._flux_to_dnde(e_ref, table, model, pwl_approx)
+            table = self._flux_to_dnde(energy_ref, table, model, pwl_approx)
 
         elif self.sed_type == "dnde" and sed_type == "e2dnde":
-            table = self._dnde_to_e2dnde(self.e_ref, table)
+            table = self._dnde_to_e2dnde(self.energy_ref, table)
 
         elif self.sed_type == "e2dnde" and sed_type == "dnde":
-            table = self._e2dnde_to_dnde(self.e_ref, table)
+            table = self._e2dnde_to_dnde(self.energy_ref, table)
 
         elif self.sed_type == "likelihood" and sed_type in ["dnde", "flux", "eflux"]:
             for suffix in ["", "_ul", "_err", "_errp", "_errn"]:
@@ -395,6 +394,10 @@ class FluxPoints:
                     )
                 except KeyError:
                     continue
+        elif self.sed_type == sed_type:
+            # do nothing if the sed type is the same
+            pass
+
         else:
             raise NotImplementedError
 
@@ -402,36 +405,36 @@ class FluxPoints:
         return FluxPoints(table)
 
     @staticmethod
-    def _e_ref_lafferty(model, e_min, e_max):
+    def _energy_ref_lafferty(model, energy_min, energy_max):
         """Helper for `to_sed_type`.
 
-        Compute e_ref that the value at e_ref corresponds
-        to the mean value between e_min and e_max.
+        Compute energy_ref that the value at energy_ref corresponds
+        to the mean value between energy_min and energy_max.
         """
-        flux = model.integral(e_min, e_max)
-        dnde_mean = flux / (e_max - e_min)
+        flux = model.integral(energy_min, energy_max)
+        dnde_mean = flux / (energy_max - energy_min)
         return model.inverse(dnde_mean)
 
     @staticmethod
-    def _dnde_from_flux(flux, model, e_ref, e_min, e_max, pwl_approx):
+    def _dnde_from_flux(flux, model, energy_ref, energy_min, energy_max, pwl_approx):
         """Helper for `to_sed_type`.
 
         Compute dnde under the assumption that flux equals expected
         flux from model.
         """
-        dnde_model = model(e_ref)
+        dnde_model = model(energy_ref)
 
         if pwl_approx:
-            index = model.spectral_index(e_ref)
+            index = model.spectral_index(energy_ref)
             flux_model = PowerLawSpectralModel.evaluate_integral(
-                emin=e_min,
-                emax=e_max,
+                energy_min=energy_min,
+                energy_max=energy_max,
                 index=index,
-                reference=e_ref,
+                reference=energy_ref,
                 amplitude=dnde_model,
             )
         else:
-            flux_model = model.integral(e_min, e_max, intervals=True)
+            flux_model = model.integral(energy_min, energy_max)
 
         return dnde_model * (flux / flux_model)
 
@@ -481,10 +484,10 @@ class FluxPoints:
     def _plot_get_energy_err(self):
         """Compute energy error for given sed type"""
         try:
-            e_min = self.table["e_min"].quantity
-            e_max = self.table["e_max"].quantity
-            e_ref = self.e_ref
-            x_err = ((e_ref - e_min), (e_max - e_ref))
+            energy_min = self.energy_min
+            energy_max = self.energy_max
+            energy_ref = self.energy_ref
+            x_err = ((energy_ref - energy_min), (energy_max - energy_ref))
         except KeyError:
             x_err = None
         return x_err
@@ -514,57 +517,57 @@ class FluxPoints:
             return np.isnan(self.table[self.sed_type])
 
     @property
-    def e_ref(self):
+    def energy_ref(self):
         """Reference energy.
 
-        Defined by `e_ref` column in `FluxPoints.table` or computed as log
-        center, if `e_min` and `e_max` columns are present in `FluxPoints.table`.
+        Defined by `energy_ref` column in `FluxPoints.table` or computed as log
+        center, if `energy_min` and `energy_max` columns are present in `FluxPoints.table`.
 
         Returns
         -------
-        e_ref : `~astropy.units.Quantity`
+        energy_ref : `~astropy.units.Quantity`
             Reference energy.
         """
         try:
             return self.table["e_ref"].quantity
         except KeyError:
-            return np.sqrt(self.e_min * self.e_max)
+            return np.sqrt(self.energy_min * self.energy_max)
 
     @property
-    def e_edges(self):
+    def energy_edges(self):
         """Edges of the energy bin.
 
         Returns
         -------
-        e_edges : `~astropy.units.Quantity`
+        energy_edges : `~astropy.units.Quantity`
             Energy edges.
         """
-        e_edges = list(self.e_min)
-        e_edges += [self.e_max[-1]]
-        return u.Quantity(e_edges, self.e_min.unit, copy=False)
+        energy_edges = list(self.energy_min)
+        energy_edges += [self.energy_max[-1]]
+        return u.Quantity(energy_edges, self.energy_min.unit, copy=False)
 
     @property
-    def e_min(self):
+    def energy_min(self):
         """Lower bound of energy bin.
 
-        Defined by `e_min` column in `FluxPoints.table`.
+        Defined by `energy_min` column in `FluxPoints.table`.
 
         Returns
         -------
-        e_min : `~astropy.units.Quantity`
+        energy_min : `~astropy.units.Quantity`
             Lower bound of energy bin.
         """
         return self.table["e_min"].quantity
 
     @property
-    def e_max(self):
+    def energy_max(self):
         """Upper bound of energy bin.
 
-        Defined by ``e_max`` column in ``table``.
+        Defined by ``energy_max`` column in ``table``.
 
         Returns
         -------
-        e_max : `~astropy.units.Quantity`
+        energy_max : `~astropy.units.Quantity`
             Upper bound of energy bin.
         """
         return self.table["e_max"].quantity
@@ -601,7 +604,7 @@ class FluxPoints:
         y_unit = u.Unit(flux_unit or DEFAULT_UNIT[sed_type])
 
         y = self.table[sed_type].quantity.to(y_unit)
-        x = self.e_ref.to(energy_unit)
+        x = self.energy_ref.to(energy_unit)
 
         # get errors and ul
         is_ul = self.is_ul
@@ -609,8 +612,8 @@ class FluxPoints:
         y_err_all = self._plot_get_flux_err(sed_type)
 
         # handle energy power
-        e_unit = self._get_y_energy_unit(y_unit)
-        y_unit = y.unit * e_unit ** energy_power
+        energy_unit_y = self._get_y_energy_unit(y_unit)
+        y_unit = y.unit * energy_unit_y ** energy_power
         y = (y * np.power(x, energy_power)).to(y_unit)
 
         y_err, x_err = None, None
@@ -631,6 +634,7 @@ class FluxPoints:
         kwargs.setdefault("marker", "+")
         kwargs.setdefault("ls", "None")
 
+        print(x_err)
         ebar = ax.errorbar(
             x[~is_ul].value, y[~is_ul].value, yerr=y_err, xerr=x_err, **kwargs
         )
@@ -715,7 +719,7 @@ class FluxPoints:
             )
             y_values = u.Quantity(y_values, y_unit, copy=False)
 
-        x = self.e_edges.to(energy_unit)
+        x = self.energy_edges.to(energy_unit)
 
         # Compute fit statistic "image" one energy bin at a time
         # by interpolating e2dnde at the log bin centers
@@ -749,7 +753,7 @@ class FluxPoints:
         return ax
 
 
-class FluxPointsEstimator(FluxEstimator):
+class FluxPointsEstimator(Estimator):
     """Flux points estimator.
 
     Estimates flux points for a given list of datasets, energies and spectral model.
@@ -768,7 +772,7 @@ class FluxPointsEstimator(FluxEstimator):
 
     Parameters
     ----------
-    e_edges : `~astropy.units.Quantity`
+    energy_edges : `~astropy.units.Quantity`
         Energy edges of the flux point bins.
     source : str or int
         For which source in the model to compute the flux points.
@@ -780,168 +784,141 @@ class FluxPointsEstimator(FluxEstimator):
         Number of norm values used for the fit statistic profile.
     norm_values : `numpy.ndarray`
         Array of norm values to be used for the fit statistic profile.
-    sigma : int
-        Sigma to use for asymmetric error computation.
-    sigma_ul : int
-        Sigma to use for upper limit computation.
+    n_sigma : int
+        Number of sigma to use for asymmetric error computation. Default is 1.
+    n_sigma_ul : int
+        Number of sigma to use for upper limit computation. Default is 2.
     reoptimize : bool
         Re-optimize other free model parameters.
+    selection_optional : list of str
+        Which additional quantities to estimate. Available options are:
+
+            * "errn-errp": estimate asymmetric errors on flux.
+            * "ul": estimate upper limits.
+            * "norm-scan": estimate fit statistic profiles.
+
+        By default all steps are executed.
     """
 
+    tag = "FluxPointsEstimator"
+    _available_selection_optional = ["errn-errp", "ul", "scan"]
+
     def __init__(
         self,
-        e_edges,
+        energy_edges=[1, 10] * u.TeV,
         source=0,
         norm_min=0.2,
         norm_max=5,
         norm_n_values=11,
         norm_values=None,
-        sigma=1,
-        sigma_ul=2,
+        n_sigma=1,
+        n_sigma_ul=2,
         reoptimize=False,
+        selection_optional="all",
     ):
-        self.e_edges = e_edges
-        super().__init__(
-            source,
-            e_edges[:2],
-            norm_min,
-            norm_max,
-            norm_n_values,
-            norm_values,
-            sigma,
-            sigma_ul,
-            reoptimize,
+        self.energy_edges = energy_edges
+        self.source = source
+        self.norm_min = norm_min
+        self.norm_max = norm_max
+        self.norm_n_values = norm_n_values
+        self.norm_values = norm_values
+        self.n_sigma = n_sigma
+        self.n_sigma_ul = n_sigma_ul
+        self.reoptimize = reoptimize
+        self.selection_optional = selection_optional
+
+    def _flux_estimator(self, energy_min, energy_max):
+        return FluxEstimator(
+            source=self.source,
+            energy_min=energy_min,
+            energy_max=energy_max,
+            norm_min=self.norm_min,
+            norm_max=self.norm_max,
+            norm_n_values=self.norm_n_values,
+            norm_values=self.norm_values,
+            n_sigma=self.n_sigma,
+            n_sigma_ul=self.n_sigma_ul,
+            reoptimize=self.reoptimize,
+            selection_optional=self.selection_optional,
         )
-        self._contribute_to_stat = False
 
-    def _freeze_empty_background(self):
-        counts_all = self._estimate_counts()["counts"]
-
-        for counts, dataset in zip(counts_all, self.datasets):
-            if isinstance(dataset, MapDataset) and counts == 0:
-                if dataset.background_model is not None:
-                    dataset.background_model.parameters.freeze_all()
-
-    @property
-    def e_groups(self):
-        """Energy grouping table `~astropy.table.Table`"""
-        dataset = self.datasets[0]
-        energy_axis = dataset.counts.geom.get_axis_by_name("energy")
-        return energy_axis.group_table(self.e_edges)
-
-    def __str__(self):
-        s = f"{self.__class__.__name__}:\n"
-        s += str(self.e_edges) + "\n"
-        return s
-
-    def run(self, datasets, steps="all"):
+    def run(self, datasets):
         """Run the flux point estimator for all energy groups.
 
         Parameters
         ----------
-        datasets : list of `~gammapy.spectrum.SpectrumDataset`
-            Spectrum datasets.
-        steps : list of str
-            Which steps to execute. See `estimate_flux_point` for details
-            and available options.
+        datasets : list of `~gammapy.datasets.Dataset`
+            Datasets
 
         Returns
         -------
         flux_points : `FluxPoints`
             Estimated flux points.
         """
-        datasets = self._check_datasets(datasets)
-
-        if not datasets.is_all_same_type or not datasets.is_all_same_energy_shape:
-            raise ValueError(
-                "Flux point estimation requires a list of datasets"
-                " of the same type and data shape."
-            )
-        self.datasets = datasets.copy()
+        datasets = Datasets(datasets).copy()
 
         rows = []
-        for e_group in self.e_groups:
-            if e_group["bin_type"].strip() != "normal":
-                log.debug("Skipping under-/ overflow bin in flux point estimation.")
-                continue
 
-            row = self._estimate_flux_point(e_group, steps=steps)
+        for energy_min, energy_max in zip(
+            self.energy_edges[:-1], self.energy_edges[1:]
+        ):
+            row = self.estimate_flux_point(
+                datasets, energy_min=energy_min, energy_max=energy_max
+            )
             rows.append(row)
 
         table = table_from_row_data(rows=rows, meta={"SED_TYPE": "likelihood"})
+
+        # TODO: this should be changed once likelihood is fully supported
         return FluxPoints(table).to_sed_type("dnde")
 
-    def _energy_mask(self, e_group, dataset):
-        energy_mask = np.zeros(dataset.data_shape)
-        energy_mask[e_group["idx_min"] : e_group["idx_max"] + 1] = 1
-        return energy_mask.astype(bool)
-
-    def _estimate_flux_point(self, e_group, steps="all"):
+    def estimate_flux_point(self, datasets, energy_min, energy_max):
         """Estimate flux point for a single energy group.
 
         Parameters
         ----------
-        e_group : `~astropy.table.Row`
-            Energy group to compute the flux point for.
-        steps : list of str
-            Which steps to execute. Available options are:
-
-                * "norm-err": estimate symmetric error.
-                * "errn-errp": estimate asymmetric errors.
-                * "ul": estimate upper limits.
-                * "ts": estimate ts and sqrt(ts) values.
-                * "norm-scan": estimate fit statistic profiles.
-
-            By default all steps are executed.
+        datasets : Datasets
+            Datasets
+        energy_min, energy_max : `~astropy.units.Quantity`
+            Energy bounds to compute the flux point for.
 
         Returns
         -------
         result : dict
             Dict with results for the flux point.
         """
-        e_min, e_max = e_group["energy_min"], e_group["energy_max"]
-        self.energy_range = [e_min, e_max]
+        result = self.estimate_counts(
+            datasets, energy_min=energy_min, energy_max=energy_max
+        )
+        fe = self._flux_estimator(energy_min=energy_min, energy_max=energy_max)
 
-        for dataset in self.datasets:
-            dataset.mask_fit = self._energy_mask(e_group=e_group, dataset=dataset)
-            mask = dataset.mask_fit
+        result.update(fe.run(datasets=datasets))
 
-            if dataset.mask_safe is not None:
-                mask &= dataset.mask_safe
-
-            self._contribute_to_stat |= mask.any()
-
-        if not self._contribute_to_stat:
-            model = self.datasets[0].models[self.source].spectral_model
-            result = self._return_nan_result(model, steps=steps)
-            result.update(self._estimate_counts())
-            return result
-
-        with self.datasets.parameters.restore_values:
-
-            self._freeze_empty_background()
-
-            result = super().run(self.datasets, steps=steps)
-            result.update(self._estimate_counts())
         return result
 
-    def _estimate_counts(self):
+    @staticmethod
+    def estimate_counts(datasets, energy_min, energy_max):
         """Estimate counts for the flux point.
 
+        Parameters
+        ----------
+        datasets : Datasets
+            Datasets
+        energy_min, energy_max : `~astropy.units.Quantity`
+            Energy bounds to compute the flux point for.
+
         Returns
         -------
         result : dict
             Dict with an array with one entry per dataset with counts for the flux point.
         """
-        if not self._contribute_to_stat:
-            return {"counts": np.zeros(len(self.datasets))}
-
         counts = []
-        for dataset in self.datasets:
-            mask = dataset.mask_fit
-            if dataset.mask_safe is not None:
-                mask &= dataset.mask_safe
 
+        for dataset in datasets:
+            energy_mask = dataset.counts.geom.energy_mask(
+                energy_min=energy_min, energy_max=energy_max, round_to_edge=True
+            )
+            mask = dataset.mask & energy_mask
             counts.append(dataset.counts.data[mask].sum())
 
         return {"counts": np.array(counts, dtype=int)}
