@@ -5,8 +5,7 @@ from astropy.coordinates import Angle
 from astropy.io import fits
 from astropy.table import Table
 from astropy.utils import lazyproperty
-from gammapy.maps import MapAxis
-from gammapy.utils.array import array_stats_str
+from gammapy.maps import MapAxes, MapAxis
 from gammapy.utils.interpolation import ScaledRegularGridInterpolator
 from gammapy.utils.scripts import make_path
 from .psf_table import EnergyDependentTablePSF, TablePSF
@@ -21,16 +20,12 @@ class PSF3D:
 
     Parameters
     ----------
-    energy_lo : `~astropy.units.Quantity`
-        Energy bins lower edges (1-dim)
-    energy_hi : `~astropy.units.Quantity`
-        Energy bins upper edges (1-dim)
-    offset : `~astropy.coordinates.Angle`
-        Offset angle (1-dim)
-    rad_lo : `~astropy.coordinates.Angle`
-        Offset angle bins lower edges
-    rad_hi : `~astropy.coordinates.Angle`
-        Offset angle bins upper edges
+    energy_axis_true : `MapAxis`
+        True energy axis.
+    offset_axis : `MapAxis`
+        Offset axis
+    rad_axis : `MapAxis`
+        Rad axis
     psf_value : `~astropy.units.Quantity`
         PSF (3-dim with axes: psf[rad_index, offset_index, energy_index]
     energy_thresh_lo : `~astropy.units.Quantity`
@@ -39,69 +34,66 @@ class PSF3D:
         Upper energy threshold.
     """
 
+    tag = "psf_table"
+
     def __init__(
         self,
-        energy_lo,
-        energy_hi,
-        offset,
-        rad_lo,
-        rad_hi,
+        energy_axis_true,
+        offset_axis,
+        rad_axis,
         psf_value,
         energy_thresh_lo=u.Quantity(0.1, "TeV"),
         energy_thresh_hi=u.Quantity(100, "TeV"),
         interp_kwargs=None,
     ):
-        self.energy_lo = energy_lo.to("TeV")
-        self.energy_hi = energy_hi.to("TeV")
-        self.offset = Angle(offset)
-        self.rad_lo = Angle(rad_lo)
-        self.rad_hi = Angle(rad_hi)
+        assert energy_axis_true.name == "energy_true"
+        assert offset_axis.name == "offset"
+        assert rad_axis.name == "rad"
+
+        assert psf_value.shape == (
+            energy_axis_true.nbin,
+            offset_axis.nbin,
+            rad_axis.nbin,
+        )
+
+        self._energy_axis_true = energy_axis_true
+        self._offset_axis = offset_axis
+        self._rad_axis = rad_axis
         self.psf_value = psf_value.to("sr^-1")
         self.energy_thresh_lo = energy_thresh_lo.to("TeV")
         self.energy_thresh_hi = energy_thresh_hi.to("TeV")
 
         self._interp_kwargs = interp_kwargs or {}
 
+    @property
+    def energy_axis_true(self):
+        return self._energy_axis_true
+
+    @property
+    def rad_axis(self):
+        return self._rad_axis
+
+    @property
+    def offset_axis(self):
+        return self._offset_axis
+
     @lazyproperty
     def _interpolate(self):
-        energy = self._energy_logcenter()
-        offset = self.offset.to("deg")
-        rad = self._rad_center()
+        energy = self.energy_axis_true.center
+        offset = self.offset_axis.center
+        rad = self.rad_axis.center
 
         return ScaledRegularGridInterpolator(
-            points=(rad, offset, energy), values=self.psf_value, **self._interp_kwargs
+            points=(energy, offset, rad), values=self.psf_value, **self._interp_kwargs
         )
 
-    def info(self):
+    def __repr__(self):
         """Print some basic info.
         """
-        ss = "\nSummary PSF3D info\n"
-        ss += "---------------------\n"
-        ss += array_stats_str(self.energy_lo, "energy_lo")
-        ss += array_stats_str(self.energy_hi, "energy_hi")
-        ss += array_stats_str(self.offset, "offset")
-        ss += array_stats_str(self.rad_lo, "rad_lo")
-        ss += array_stats_str(self.rad_hi, "rad_hi")
-        ss += array_stats_str(self.psf_value, "psf_value")
-
-        # TODO: should quote containment values also
-
-        return ss
-
-    def _energy_logcenter(self):
-        """Get logcenters of energy bins.
-
-        Returns
-        -------
-        energies : `~astropy.units.Quantity`
-            Logcenters of energy bins
-        """
-        return np.sqrt(self.energy_lo * self.energy_hi)
-
-    def _rad_center(self):
-        """Get centers of rad bins (`~astropy.coordinates.Angle` in deg).
-        """
-        return ((self.rad_hi + self.rad_lo) / 2).to("deg")
+        info = self.__class__.__name__ + "\n"
+        info += "-" * len(self.__class__.__name__) + "\n\n"
+        info += f"\tshape      : {self.psf_value.shape}\n"
+        return info
 
     @classmethod
     def read(cls, filename, hdu="PSF_2D_TABLE"):
@@ -126,18 +118,7 @@ class PSF3D:
         table : `~astropy.table.Table`
             Table Table-PSF info.
         """
-        theta_lo = table["THETA_LO"].quantity[0]
-        theta_hi = table["THETA_HI"].quantity[0]
-        offset = (theta_hi + theta_lo) / 2
-        offset = Angle(offset, unit=table["THETA_LO"].unit)
-
-        energy_lo = table["ENERG_LO"].quantity[0]
-        energy_hi = table["ENERG_HI"].quantity[0]
-
-        rad_lo = table["RAD_LO"].quantity[0]
-        rad_hi = table["RAD_HI"].quantity[0]
-
-        psf_value = table["RPSF"].quantity[0]
+        psf_value = table["RPSF"].quantity[0].transpose()
 
         opts = {}
         try:
@@ -146,42 +127,34 @@ class PSF3D:
         except KeyError:
             pass
 
-        return cls(energy_lo, energy_hi, offset, rad_lo, rad_hi, psf_value, **opts)
+        energy_axis_true = MapAxis.from_table(
+            table, column_prefix="ENERG", format="gadf-dl3"
+        )
+        offset_axis = MapAxis.from_table(
+            table, column_prefix="THETA", format="gadf-dl3"
+        )
+        rad_axis = MapAxis.from_table(table, column_prefix="RAD", format="gadf-dl3")
 
-    def to_fits(self):
-        """
-        Convert PSF table data to FITS HDU list.
+        return cls(
+            energy_axis_true=energy_axis_true,
+            offset_axis=offset_axis,
+            rad_axis=rad_axis,
+            psf_value=psf_value,
+            **opts,
+        )
+
+    def to_hdulist(self):
+        """Convert PSF table data to FITS HDU list.
 
         Returns
         -------
         hdu_list : `~astropy.io.fits.HDUList`
             PSF in HDU list format.
         """
-        # Set up data
-        names = [
-            "ENERG_LO",
-            "ENERG_HI",
-            "THETA_LO",
-            "THETA_HI",
-            "RAD_LO",
-            "RAD_HI",
-            "RPSF",
-        ]
-        units = ["TeV", "TeV", "deg", "deg", "deg", "deg", "sr^-1"]
-        data = [
-            self.energy_lo,
-            self.energy_hi,
-            self.offset,
-            self.offset,
-            self.rad_lo,
-            self.rad_hi,
-            self.psf_value,
-        ]
+        axes = MapAxes([self.offset_axis, self.energy_axis_true, self.rad_axis])
+        table = axes.to_table(format="gadf-dl3")
 
-        table = Table()
-        for name_, data_, unit_ in zip(names, data, units):
-            table[name_] = [data_]
-            table[name_].unit = unit_
+        table["RPSF"] = self.psf_value.T[np.newaxis]
 
         hdu = fits.BinTableHDU(table)
         hdu.header["LO_THRES"] = self.energy_thresh_lo.value
@@ -194,7 +167,7 @@ class PSF3D:
 
         Calls `~astropy.io.fits.HDUList.writeto`, forwarding all arguments.
         """
-        self.to_fits().writeto(filename, *args, **kwargs)
+        self.to_hdulist().writeto(str(make_path(filename)), *args, **kwargs)
 
     def evaluate(self, energy=None, offset=None, rad=None):
         """Interpolate PSF value at a given offset and energy.
@@ -214,20 +187,20 @@ class PSF3D:
             Interpolated value
         """
         if energy is None:
-            energy = self._energy_logcenter()
+            energy = self.energy_axis_true.center
         if offset is None:
-            offset = self.offset
+            offset = self.offset_axis.center
         if rad is None:
-            rad = self._rad_center()
+            rad = self.rad_axis.center
 
         rad = np.atleast_1d(u.Quantity(rad))
         offset = np.atleast_1d(u.Quantity(offset))
         energy = np.atleast_1d(u.Quantity(energy))
         return self._interpolate(
             (
-                rad[:, np.newaxis, np.newaxis],
-                offset[np.newaxis, :, np.newaxis],
                 energy[np.newaxis, np.newaxis, :],
+                offset[np.newaxis, :, np.newaxis],
+                rad[:, np.newaxis, np.newaxis],
             )
         )
 
@@ -252,16 +225,18 @@ class PSF3D:
             Energy-dependent PSF
         """
         theta = Angle(theta)
-        energies = self._energy_logcenter()
 
-        if rad is None:
-            rad = self._rad_center()
+        if rad is not None:
+            rad_axis = MapAxis.from_edges(rad, name="rad")
         else:
-            rad = Angle(rad)
+            rad_axis = self.rad_axis
 
-        psf_value = self.evaluate(offset=theta, rad=rad).squeeze()
+        psf_value = self.evaluate(offset=theta, rad=rad_axis.center).squeeze()
         return EnergyDependentTablePSF(
-            energy=energies, rad=rad, exposure=exposure, psf_value=psf_value.T
+            energy_axis_true=self.energy_axis_true,
+            rad_axis=rad_axis,
+            exposure=exposure,
+            psf_value=psf_value.transpose(),
         )
 
     def to_table_psf(self, energy, theta="0 deg", **kwargs):
@@ -282,8 +257,7 @@ class PSF3D:
         energy = u.Quantity(energy)
         theta = Angle(theta)
         psf_value = self.evaluate(energy, theta).squeeze()
-        rad = self._rad_center()
-        return TablePSF(rad, psf_value, **kwargs)
+        return TablePSF(rad_axis=self.rad_axis, psf_value=psf_value, **kwargs)
 
     def containment_radius(
         self, energy, theta="0 deg", fraction=0.68, interp_kwargs=None
@@ -315,7 +289,7 @@ class PSF3D:
         return u.Quantity(radii).T.squeeze()
 
     def plot_containment_vs_energy(
-        self, fractions=[0.68, 0.95], thetas=Angle([0, 1], "deg"), ax=None
+        self, fractions=[0.68, 0.95], thetas=Angle([0, 1], "deg"), ax=None, **kwargs
     ):
         """Plot containment fraction as a function of energy.
         """
@@ -324,14 +298,14 @@ class PSF3D:
         ax = plt.gca() if ax is None else ax
 
         energy = MapAxis.from_energy_bounds(
-            self.energy_lo[0], self.energy_hi[-1], 100
+            self.energy_axis_true.edges[0], self.energy_axis_true.edges[-1], 100
         ).edges
 
         for theta in thetas:
             for fraction in fractions:
                 radius = self.containment_radius(energy, theta, fraction)
-                label = f"{theta.deg} deg, {100 * fraction:.1f}%"
-                ax.plot(energy.value, radius.value, label=label)
+                kwargs.setdefault("label", f"{theta.deg} deg, {100 * fraction:.1f}%")
+                ax.plot(energy.value, radius.value, **kwargs)
 
         ax.semilogx()
         ax.legend(loc="best")
@@ -352,11 +326,8 @@ class PSF3D:
         table = self.to_table_psf(energy=energy, theta=theta)
         return table.plot_psf_vs_rad()
 
-    def plot_containment(
-        self, fraction=0.68, ax=None, show_safe_energy=False, add_cbar=True, **kwargs
-    ):
-        """
-        Plot containment image with energy and theta axes.
+    def plot_containment(self, fraction=0.68, ax=None, add_cbar=True, **kwargs):
+        """Plot containment image with energy and theta axes.
 
         Parameters
         ----------
@@ -369,8 +340,8 @@ class PSF3D:
 
         ax = plt.gca() if ax is None else ax
 
-        energy = self._energy_logcenter()
-        offset = self.offset
+        energy = self.energy_axis_true.center
+        offset = self.offset_axis.center
 
         # Set up and compute data
         containment = self.containment_radius(energy, offset, fraction)
@@ -392,8 +363,10 @@ class PSF3D:
         ax.set_xlim(x.min(), x.max())
         ax.set_ylim(y.min(), y.max())
 
-        if show_safe_energy:
+        try:
             self._plot_safe_energy_range(ax)
+        except KeyError:
+            pass
 
         if add_cbar:
             label = f"Containment radius R{100 * fraction:.0f} ({containment.unit})"
@@ -404,9 +377,9 @@ class PSF3D:
     def _plot_safe_energy_range(self, ax):
         """add safe energy range lines to the plot"""
         esafe = self.energy_thresh_lo
-        omin = self.offset.value.min()
-        omax = self.offset.value.max()
-        ax.hlines(y=esafe.value, xmin=omin, xmax=omax)
+        omin = self.offset_axis.center.value.min()
+        omax = self.offset_axis.center.value.max()
+        ax.vlines(x=esafe.value, ymin=omin, ymax=omax)
         label = f"Safe energy threshold: {esafe:3.2f}"
         ax.text(x=0.1, y=0.9 * esafe.value, s=label, va="top")
 
