@@ -5,6 +5,7 @@ import scipy.integrate
 from astropy import units as u
 from astropy.coordinates import Angle
 from astropy.io import fits
+from astropy.table import Table
 from astropy.utils import lazyproperty
 from gammapy.maps import MapAxis
 from gammapy.utils.array import array_stats_str
@@ -22,7 +23,7 @@ class TablePSF:
 
     Parameters
     ----------
-    rad : `~astropy.units.Quantity` with angle units
+    rad_axis : `~astropy.units.Quantity` with angle units
         Offset wrt source position
     psf_value : `~astropy.units.Quantity` with sr^-1 units
         PSF value array
@@ -30,25 +31,31 @@ class TablePSF:
         Keyword arguments passed to `ScaledRegularGridInterpolator`
     """
 
-    def __init__(self, rad, psf_value, interp_kwargs=None):
-        self.rad = Angle(rad).to("rad")
+    def __init__(self, rad_axis, psf_value, interp_kwargs=None):
+        assert rad_axis.name == "rad"
+        self._rad_axis = rad_axis
+
         self.psf_value = u.Quantity(psf_value).to("sr^-1")
 
         self._interp_kwargs = interp_kwargs or {}
 
+    @property
+    def rad_axis(self):
+        return self._rad_axis
+
     @lazyproperty
     def _interpolate(self):
-        points = (self.rad,)
+        points = (self.rad_axis.center,)
         return ScaledRegularGridInterpolator(
             points=points, values=self.psf_value, **self._interp_kwargs
         )
 
     @lazyproperty
     def _interpolate_containment(self):
-        if self.rad[0] > 0:
-            rad = self.rad.insert(0, 0)
-        else:
-            rad = self.rad
+        rad = self.rad_axis.center
+
+        if rad[0] > 0:
+            rad = rad.insert(0, 0)
 
         rad_drad = 2 * np.pi * rad * self.evaluate(rad)
         values = scipy.integrate.cumtrapz(
@@ -98,13 +105,13 @@ class TablePSF:
             raise ValueError(f"Invalid shape: {shape}")
 
         psf_value = u.Quantity(psf_value, "sr^-1")
-
-        return cls(rad, psf_value)
+        rad_axis = MapAxis.from_nodes(rad, name="rad")
+        return cls(rad_axis=rad_axis, psf_value=psf_value)
 
     def info(self):
         """Print basic info."""
-        ss = array_stats_str(self.rad.deg, "offset")
-        ss += f"integral = {self.integral()}\n"
+        ss = array_stats_str(self.rad_axis.center, "offset")
+        ss += f"integral = {self.containment(self.rad_axis.edges[-1])}\n"
 
         for containment in [68, 80, 95]:
             radius = self.containment_radius(0.01 * containment)
@@ -164,7 +171,12 @@ class TablePSF:
         rad : `~astropy.coordinates.Angle`
             Containment radius angle
         """
-        rad_max = Angle(np.linspace(0, self.rad[-1].value, 10 * len(self.rad)), "rad")
+        # TODO: check whether starting
+        rad_max = Angle(
+            np.linspace(0 * u.deg, self.rad_axis.center[-1], 10 * self.rad_axis.nbin),
+            "rad",
+        )
+
         containment = self.containment(rad_max=rad_max)
 
         fraction = np.atleast_1d(fraction)
@@ -178,32 +190,9 @@ class TablePSF:
         Computes the total PSF integral via the :math:`dP / dr` spline
         and then divides the :math:`dP / dr` array.
         """
-        integral = self.containment(self.rad[-1])
+        integral = self.containment(self.rad_axis.edges[-1])
         self.psf_value /= integral
 
-    def broaden(self, factor, normalize=True):
-        r"""Broaden PSF by scaling the offset array.
-
-        For a broadening factor :math:`f` and the offset
-        array :math:`r`, the offset array scaled
-        in the following way:
-
-        .. math::
-            r_{new} = f \times r_{old}
-            \frac{dP}{dr}(r_{new}) = \frac{dP}{dr}(r_{old})
-
-        Parameters
-        ----------
-        factor : float
-            Broadening factor
-        normalize : bool
-            Normalize PSF after broadening
-        """
-        self.rad *= factor
-        self._setup_interpolators()
-        if normalize:
-            self.normalize()
-
     def plot_psf_vs_rad(self, ax=None, **kwargs):
         """Plot PSF vs radius.
 
@@ -218,7 +207,11 @@ class TablePSF:
 
         ax = plt.gca() if ax is None else ax
 
-        ax.plot(self.rad.to_value("deg"), self.psf_value.to_value("sr-1"), **kwargs)
+        ax.plot(
+            self.rad_axis.center.to_value("deg"),
+            self.psf_value.to_value("sr-1"),
+            **kwargs,
+        )
         ax.set_yscale("log")
         ax.set_xlabel("Radius (deg)")
         ax.set_ylabel("PSF (sr-1)")
@@ -231,10 +224,10 @@ class EnergyDependentTablePSF:
 
     Parameters
     ----------
-    energy : `~astropy.units.Quantity`
-        Energy (1-dim)
-    rad : `~astropy.units.Quantity` with angle units
-        Offset angle wrt source position (1-dim)
+    energy_axis_true : `MapAxis`
+        Energy axis
+    rad_axis : `MapAxis`
+        Offset angle wrt source position axis
     exposure : `~astropy.units.Quantity`
         Exposure (1-dim)
     psf_value : `~astropy.units.Quantity`
@@ -243,49 +236,75 @@ class EnergyDependentTablePSF:
         Interpolation keyword arguments pass to `ScaledRegularGridInterpolator`.
     """
 
-    def __init__(self, energy, rad, exposure=None, psf_value=None, interp_kwargs=None):
-        self.energy = u.Quantity(energy).to("GeV")
-        self.rad = u.Quantity(rad).to("radian")
+    def __init__(
+        self,
+        energy_axis_true,
+        rad_axis,
+        exposure=None,
+        psf_value=None,
+        interp_kwargs=None,
+    ):
+        self._rad_axis = rad_axis
+        self._energy_axis_true = energy_axis_true
+
+        assert energy_axis_true.name == "energy_true"
+        assert rad_axis.name == "rad"
+
         if exposure is None:
-            self.exposure = u.Quantity(np.ones(len(energy)), "cm^2 s")
+            self.exposure = u.Quantity(np.ones(self.energy_axis_true.nbin), "cm^2 s")
         else:
             self.exposure = u.Quantity(exposure).to("cm^2 s")
 
+        shape = (energy_axis_true.nbin, rad_axis.nbin)
         if psf_value is None:
-            self.psf_value = u.Quantity(np.zeros(len(energy), len(rad)), "sr^-1")
+            self.psf_value = np.zeros(shape) * u.Unit("sr^-1")
         else:
+            assert np.shape(psf_value) == shape
             self.psf_value = u.Quantity(psf_value).to("sr^-1")
 
         self._interp_kwargs = interp_kwargs or {}
 
+    @property
+    def energy_axis_true(self):
+        return self._energy_axis_true
+
+    @property
+    def rad_axis(self):
+        return self._rad_axis
+
     @lazyproperty
     def _interpolate(self):
-        points = (self.energy, self.rad)
+        points = (self.energy_axis_true.center, self.rad_axis.center)
         return ScaledRegularGridInterpolator(
             points=points, values=self.psf_value, **self._interp_kwargs
         )
 
     @lazyproperty
     def _interpolate_containment(self):
-        if self.rad[0] > 0:
-            rad = self.rad.insert(0, 0)
-        else:
-            rad = self.rad
+        rad = self.rad_axis.center
 
-        rad_drad = 2 * np.pi * rad * self.evaluate(energy=self.energy, rad=rad)
+        if rad[0] > 0:
+            rad = rad.insert(0, 0)
+
+        rad_drad = (
+            2
+            * np.pi
+            * rad
+            * self.evaluate(energy=self.energy_axis_true.center, rad=rad)
+        )
         values = scipy.integrate.cumtrapz(
             rad_drad.to_value("rad-1"), rad.to_value("rad"), initial=0, axis=1
         )
 
-        points = (self.energy, rad)
+        points = (self.energy_axis_true.center, rad)
         return ScaledRegularGridInterpolator(points=points, values=values, fill_value=1)
 
     def __str__(self):
         ss = "EnergyDependentTablePSF\n"
         ss += "-----------------------\n"
         ss += "\nAxis info:\n"
-        ss += "  " + array_stats_str(self.rad.to("deg"), "rad")
-        ss += "  " + array_stats_str(self.energy, "energy")
+        ss += "  " + array_stats_str(self.rad_axis.center.to("deg"), "rad")
+        ss += "  " + array_stats_str(self.energy_axis_true.center, "energy")
         ss += "\nContainment info:\n"
         # Print some example containment radii
         fractions = [0.68, 0.95]
@@ -298,7 +317,7 @@ class EnergyDependentTablePSF:
         return ss
 
     @classmethod
-    def from_fits(cls, hdu_list):
+    def from_hdulist(cls, hdu_list):
         """Create `EnergyDependentTablePSF` from ``gtpsf`` format HDU list.
 
         Parameters
@@ -306,14 +325,22 @@ class EnergyDependentTablePSF:
         hdu_list : `~astropy.io.fits.HDUList`
             HDU list with ``THETA`` and ``PSF`` extensions.
         """
+        # TODO: move this to MapAxis.from_table()
         rad = Angle(hdu_list["THETA"].data["Theta"], "deg")
+        rad_axis = MapAxis.from_nodes(rad, name="rad")
         energy = u.Quantity(hdu_list["PSF"].data["Energy"], "MeV")
+        energy_axis_true = MapAxis.from_nodes(energy, name="energy_true", interp="log")
         exposure = u.Quantity(hdu_list["PSF"].data["Exposure"], "cm^2 s")
         psf_value = u.Quantity(hdu_list["PSF"].data["PSF"], "sr^-1")
 
-        return cls(energy, rad, exposure, psf_value)
+        return cls(
+            energy_axis_true=energy_axis_true,
+            rad_axis=rad_axis,
+            exposure=exposure,
+            psf_value=psf_value,
+        )
 
-    def to_fits(self):
+    def to_hdulist(self):
         """Convert to FITS HDU list format.
 
         Returns
@@ -323,13 +350,20 @@ class EnergyDependentTablePSF:
         """
         # TODO: write HEADER keywords as gtpsf
 
-        data = self.rad
-        theta_hdu = fits.BinTableHDU(data=data, name="Theta")
+        data = Table([self.rad_axis.center.to("deg")], names=["Theta"])
+        theta_hdu = fits.BinTableHDU(data=data, name="THETA")
 
-        data = [self.energy, self.exposure, self.psf_value]
+        data = Table(
+            [
+                self.energy_axis_true.center.to("MeV"),
+                self.exposure.to("cm^2 s"),
+                self.psf_value.to("sr^-1"),
+            ],
+            names=["Energy", "Exposure", "PSF"],
+        )
         psf_hdu = fits.BinTableHDU(data=data, name="PSF")
 
-        hdu_list = fits.HDUList([theta_hdu, psf_hdu])
+        hdu_list = fits.HDUList([fits.PrimaryHDU(), theta_hdu, psf_hdu])
         return hdu_list
 
     @classmethod
@@ -341,15 +375,15 @@ class EnergyDependentTablePSF:
         filename : str
             File name
         """
-        with fits.open(make_path(filename), memmap=False) as hdulist:
-            return cls.from_fits(hdulist)
+        with fits.open(str(make_path(filename)), memmap=False) as hdulist:
+            return cls.from_hdulist(hdulist)
 
-    def write(self, *args, **kwargs):
+    def write(self, filename, *args, **kwargs):
         """Write to FITS file.
 
         Calls `~astropy.io.fits.HDUList.writeto`, forwarding all arguments.
         """
-        self.to_fits().writeto(*args, **kwargs)
+        self.to_hdulist().writeto(str(make_path(filename)), *args, **kwargs)
 
     def evaluate(self, energy=None, rad=None, method="linear"):
         """Evaluate the PSF at a given energy and offset
@@ -369,13 +403,13 @@ class EnergyDependentTablePSF:
             Interpolated value
         """
         if energy is None:
-            energy = self.energy
+            energy = self.energy_axis_true.center
 
         if rad is None:
-            rad = self.rad
+            rad = self.rad_axis.center
 
-        energy = np.atleast_1d(u.Quantity(energy))[:, np.newaxis]
-        rad = np.atleast_1d(u.Quantity(rad))
+        energy = u.Quantity(energy, ndmin=1)[:, np.newaxis]
+        rad = u.Quantity(rad, ndmin=1)
         return self._interpolate((energy, rad), clip=True, method=method)
 
     def table_psf_at_energy(self, energy, method="linear", **kwargs):
@@ -394,9 +428,11 @@ class EnergyDependentTablePSF:
             Table PSF
         """
         psf_value = self.evaluate(energy=energy, method=method)[0, :]
-        return TablePSF(self.rad, psf_value, **kwargs)
+        return TablePSF(rad_axis=self.rad_axis, psf_value=psf_value, **kwargs)
 
-    def table_psf_in_energy_band(self, energy_band, spectrum=None, n_bins=11, **kwargs):
+    def table_psf_in_energy_range(
+        self, energy_range, spectrum=None, n_bins=11, **kwargs
+    ):
         """Average PSF in a given energy band.
 
         Expected counts in sub energy bands given the given exposure
@@ -404,7 +440,7 @@ class EnergyDependentTablePSF:
 
         Parameters
         ----------
-        energy_band : `~astropy.units.Quantity`
+        energy_range : `~astropy.units.Quantity`
             Energy band
         spectrum : `~gammapy.modeling.models.SpectralModel`
             Spectral model used for weighting the PSF. Default is a power law
@@ -423,9 +459,9 @@ class EnergyDependentTablePSF:
         if spectrum is None:
             spectrum = PowerLawSpectralModel()
 
-        exposure = TemplateSpectralModel(self.energy, self.exposure)
+        exposure = TemplateSpectralModel(self.energy_axis_true.center, self.exposure)
 
-        e_min, e_max = energy_band
+        e_min, e_max = energy_range
         energy = MapAxis.from_energy_bounds(e_min, e_max, n_bins).edges
 
         weights = spectrum(energy) * exposure(energy)
@@ -433,7 +469,7 @@ class EnergyDependentTablePSF:
 
         psf_value = self.evaluate(energy=energy)
         psf_value_weighted = weights[:, np.newaxis] * psf_value
-        return TablePSF(self.rad, psf_value_weighted.sum(axis=0), **kwargs)
+        return TablePSF(self.rad_axis, psf_value_weighted.sum(axis=0), **kwargs)
 
     def containment_radius(self, energy, fraction=0.68):
         """Containment radius.
@@ -451,7 +487,7 @@ class EnergyDependentTablePSF:
             Containment radius in deg
         """
         # upsamle for better precision
-        rad_max = Angle(np.linspace(0, self.rad[-1].value, 10 * len(self.rad)), "rad")
+        rad_max = Angle(self.rad_axis.upsample(factor=10).center)
         containment = self.containment(energy=energy, rad_max=rad_max)
 
         # find nearest containment value
@@ -502,7 +538,7 @@ class EnergyDependentTablePSF:
             psf_value = np.squeeze(self.evaluate(energy=energy))
             label = f"{energy:.0f}"
             ax.plot(
-                self.rad.to_value("deg"),
+                self.rad_axis.center.to_value("deg"),
                 psf_value.to_value("sr-1"),
                 label=label,
                 **kwargs,
@@ -523,9 +559,14 @@ class EnergyDependentTablePSF:
         ax = plt.gca() if ax is None else ax
 
         for fraction in fractions:
-            rad = self.containment_radius(self.energy, fraction)
+            rad = self.containment_radius(self.energy_axis_true.center, fraction)
             label = f"{100 * fraction:.1f}% Containment"
-            ax.plot(self.energy.value, rad.value, label=label, **kwargs)
+            ax.plot(
+                self.energy_axis_true.center.to("GeV").value,
+                rad.to("deg").value,
+                label=label,
+                **kwargs,
+            )
 
         ax.semilogx()
         ax.legend(loc="best")
@@ -537,7 +578,7 @@ class EnergyDependentTablePSF:
         import matplotlib.pyplot as plt
 
         plt.figure(figsize=(4, 3))
-        plt.plot(self.energy, self.exposure, color="black", lw=3)
+        plt.plot(self.energy_axis_true.center, self.exposure, color="black", lw=3)
         plt.semilogx()
         plt.xlabel("Energy (MeV)")
         plt.ylabel("Exposure (cm^2 s)")
@@ -567,5 +608,8 @@ class EnergyDependentTablePSF:
             psf_value = np.nan_to_num(psf_value / exposure)
 
         return self.__class__(
-            energy=self.energy, rad=self.rad, psf_value=psf_value.T, exposure=exposure
+            energy_axis_true=self.energy_axis_true,
+            rad_axis=self.rad_axis,
+            psf_value=psf_value.T,
+            exposure=exposure,
         )
