@@ -1,42 +1,27 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Cube models (axes: lon, lat, energy)."""
 import copy
-from pathlib import Path
 import numpy as np
 import astropy.units as u
 from gammapy.maps import Map, MapAxis, RegionGeom, WcsGeom
-from gammapy.modeling import Covariance, Parameter, Parameters
+from gammapy.modeling import Covariance, Parameters
 from gammapy.modeling.parameter import _get_parameters_str
+from gammapy.utils.fits import LazyFitsData
 from gammapy.utils.scripts import make_name, make_path
 from .core import Model, Models
-from .spatial import SpatialModel
-from .spectral import SpectralModel
+from .spatial import ConstantSpatialModel, SpatialModel
+from .spectral import PowerLawNormSpectralModel, SpectralModel, TemplateSpectralModel
 from .temporal import TemporalModel
 
-
-class SkyModelBase(Model):
-    """Sky model base class"""
-
-    def __add__(self, other):
-        if isinstance(other, (Models, list)):
-            return Models([self, *other])
-        elif isinstance(other, (SkyModel, SkyDiffuseCube)):
-            return Models([self, other])
-        else:
-            raise TypeError(f"Invalid type: {other!r}")
-
-    def __radd__(self, model):
-        return self.__add__(model)
-
-    def __call__(self, lon, lat, energy, time=None):
-        return self.evaluate(lon, lat, energy, time)
-
-    def evaluate_geom(self, geom, gti=None):
-        coords = geom.get_coord(frame=self.frame)
-        return self(coords.lon, coords.lat, coords["energy_true"])
+__all__ = [
+    "SkyModel",
+    "FoVBackgroundModel",
+    "BackgroundModel",
+    "create_fermi_isotropic_diffuse_model",
+]
 
 
-class SkyModel(SkyModelBase):
+class SkyModel(Model):
     """Sky model component.
 
     This model represents a factorised sky model.
@@ -53,9 +38,14 @@ class SkyModel(SkyModelBase):
         Temporal model
     name : str
         Model identifier
+    apply_irf : dict
+        Dictionary declaring which IRFs should be applied to this model. Options
+        are {"exposure": True, "psf": True, "edisp": True}
+    datasets_names : list of str
+        Which datasets this model is applied to.
     """
 
-    tag = "SkyModel"
+    tag = ["SkyModel", "sky-model"]
     _apply_irf_default = {"exposure": True, "psf": True, "edisp": True}
 
     def __init__(
@@ -77,12 +67,13 @@ class SkyModel(SkyModelBase):
 
         self.apply_irf = apply_irf
         self.datasets_names = datasets_names
+        self._check_unit()
         super().__init__()
 
     @property
     def _models(self):
         models = self.spectral_model, self.spatial_model, self.temporal_model
-        return [_ for _ in models if _]
+        return [model for model in models if model is not None]
 
     def _check_covariance(self):
         if not self.parameters == self._covariance.parameters:
@@ -90,6 +81,33 @@ class SkyModel(SkyModelBase):
                 [model.covariance for model in self._models],
             )
 
+    def _check_unit(self):
+        from gammapy.data.gti import GTI
+
+        # evaluate over a test geom to check output unit
+        # TODO simpler way to test this ?
+        axis = MapAxis.from_energy_bounds(
+            "0.1 TeV", "10 TeV", nbin=1, name="energy_true"
+        )
+
+        geom = WcsGeom.create(skydir=self.position, npix=(2, 2), axes=[axis])
+
+        gti = GTI.create(1 * u.day, 2 * u.day)
+        value = self.evaluate_geom(geom, gti)
+
+        if self.apply_irf["exposure"]:
+            ref_unit = u.Unit("cm-2 s-1 MeV-1 sr-1")
+        else:
+            ref_unit = u.Unit("sr-1")
+
+        if self.spatial_model is None:
+            ref_unit = ref_unit / u.Unit("sr-1")
+
+        if not value.unit.is_equivalent(ref_unit):
+            raise ValueError(
+                f"SkyModel unit {value.unit} is not equivalent to {ref_unit}"
+            )
+
     @property
     def covariance(self):
         self._check_covariance()
@@ -147,7 +165,6 @@ class SkyModel(SkyModelBase):
     def spectral_model(self, model):
         if not (model is None or isinstance(model, SpectralModel)):
             raise TypeError(f"Invalid type: {model!r}")
-
         self._spectral_model = model
 
     @property
@@ -165,7 +182,7 @@ class SkyModel(SkyModelBase):
     @property
     def position(self):
         """`~astropy.coordinates.SkyCoord`"""
-        return self.spatial_model.position
+        return getattr(self.spatial_model, "position", None)
 
     @property
     def evaluation_radius(self):
@@ -176,6 +193,20 @@ class SkyModel(SkyModelBase):
     def frame(self):
         return self.spatial_model.frame
 
+    def __add__(self, other):
+        if isinstance(other, (Models, list)):
+            return Models([self, *other])
+        elif isinstance(other, (SkyModel, BackgroundModel)):
+            return Models([self, other])
+        else:
+            raise TypeError(f"Invalid type: {other!r}")
+
+    def __radd__(self, model):
+        return self.__add__(model)
+
+    def __call__(self, lon, lat, energy, time=None):
+        return self.evaluate(lon, lat, energy, time)
+
     def __repr__(self):
         return (
             f"{self.__class__.__name__}("
@@ -210,7 +241,12 @@ class SkyModel(SkyModelBase):
         # TODO: case if self.temporal_model is not None, introduce time in arguments ?
 
         if self.spatial_model is not None:
-            value = value * self.spatial_model(lon, lat)  # pylint:disable=not-callable
+            if self.spatial_model.is_energy_dependent:
+                spatial = self.spatial_model(lon, lat, energy)
+            else:
+                spatial = self.spatial_model(lon, lat)
+
+            value = value * spatial  # pylint:disable=not-callable
 
         if (self.temporal_model is not None) and (time is not None):
             value = value * self.temporal_model(time)
@@ -219,11 +255,11 @@ class SkyModel(SkyModelBase):
 
     def evaluate_geom(self, geom, gti=None):
         """Evaluate model on `~gammapy.maps.Geom`."""
-        energy = geom.get_axis_by_name("energy_true").center[:, np.newaxis, np.newaxis]
+        energy = geom.axes["energy_true"].center[:, np.newaxis, np.newaxis]
         value = self.spectral_model(energy)
 
         if self.spatial_model:
-            value = value * self.spatial_model.evaluate_geom(geom.to_image())
+            value = value * self.spatial_model.evaluate_geom(geom)
 
         if self.temporal_model:
             integral = self.temporal_model.integral(gti.time_start, gti.time_stop)
@@ -246,16 +282,15 @@ class SkyModel(SkyModelBase):
         flux : `Map`
             Predicted flux map
         """
-        energy = geom.get_axis_by_name("energy_true").edges
-        value = self.spectral_model.integral(
-            energy[:-1], energy[1:], intervals=True
-        ).reshape((-1, 1, 1))
+        energy = geom.axes["energy_true"].edges
+        value = self.spectral_model.integral(energy[:-1], energy[1:],).reshape(
+            (-1, 1, 1)
+        )
 
         if self.spatial_model and not isinstance(geom, RegionGeom):
             # TODO: integrate spatial model over region to correct for
             #  containment
-            geom_image = geom.to_image()
-            value = value * self.spatial_model.integrate(geom_image).quantity
+            value = value * self.spatial_model.integrate_geom(geom).quantity
 
         if self.temporal_model:
             integral = self.temporal_model.integral(gti.time_start, gti.time_stop)
@@ -284,18 +319,18 @@ class SkyModel(SkyModelBase):
 
         return self.__class__(**kwargs)
 
-    def to_dict(self):
+    def to_dict(self, full_output=False):
         """Create dict for YAML serilisation"""
         data = {}
         data["name"] = self.name
-        data["type"] = self.tag
-        data["spectral"] = self.spectral_model.to_dict()
+        data["type"] = self.tag[0]
+        data["spectral"] = self.spectral_model.to_dict(full_output)
 
         if self.spatial_model is not None:
-            data["spatial"] = self.spatial_model.to_dict()
+            data["spatial"] = self.spatial_model.to_dict(full_output)
 
         if self.temporal_model is not None:
-            data["temporal"] = self.temporal_model.to_dict()
+            data["temporal"] = self.temporal_model.to_dict(full_output)
 
         if self.apply_irf != self._apply_irf_default:
             data["apply_irf"] = self.apply_irf
@@ -309,18 +344,18 @@ class SkyModel(SkyModelBase):
     def from_dict(cls, data):
         """Create SkyModel from dict"""
         from gammapy.modeling.models import (
-            SPATIAL_MODELS,
-            SPECTRAL_MODELS,
-            TEMPORAL_MODELS,
+            SPATIAL_MODEL_REGISTRY,
+            SPECTRAL_MODEL_REGISTRY,
+            TEMPORAL_MODEL_REGISTRY,
         )
 
-        model_class = SPECTRAL_MODELS.get_cls(data["spectral"]["type"])
+        model_class = SPECTRAL_MODEL_REGISTRY.get_cls(data["spectral"]["type"])
         spectral_model = model_class.from_dict(data["spectral"])
 
         spatial_data = data.get("spatial")
 
         if spatial_data is not None:
-            model_class = SPATIAL_MODELS.get_cls(spatial_data["type"])
+            model_class = SPATIAL_MODEL_REGISTRY.get_cls(spatial_data["type"])
             spatial_model = model_class.from_dict(spatial_data)
         else:
             spatial_model = None
@@ -328,7 +363,7 @@ class SkyModel(SkyModelBase):
         temporal_data = data.get("temporal")
 
         if temporal_data is not None:
-            model_class = TEMPORAL_MODELS.get_cls(temporal_data["type"])
+            model_class = TEMPORAL_MODEL_REGISTRY.get_cls(temporal_data["type"])
             temporal_model = model_class.from_dict(temporal_data)
         else:
             temporal_model = None
@@ -343,23 +378,26 @@ class SkyModel(SkyModelBase):
         )
 
     def __str__(self):
-        str_ = self.__class__.__name__ + "\n\n"
+        str_ = f"{self.__class__.__name__}\n\n"
+
         str_ += "\t{:26}: {}\n".format("Name", self.name)
 
         str_ += "\t{:26}: {}\n".format("Datasets names", self.datasets_names)
 
-        str_ += "\t{:26}: {}\n".format("Spectral model type", self.spectral_model.tag)
+        str_ += "\t{:26}: {}\n".format(
+            "Spectral model type", self.spectral_model.__class__.__name__
+        )
 
         if self.spatial_model is not None:
-            spatial_type = self.spatial_model.tag
+            spatial_type = self.spatial_model.__class__.__name__
         else:
-            spatial_type = "None"
+            spatial_type = ""
         str_ += "\t{:26}: {}\n".format("Spatial  model type", spatial_type)
 
         if self.temporal_model is not None:
-            temporal_type = self.temporal_model.tag
+            temporal_type = self.temporal_model.__class__.__name__
         else:
-            temporal_type = "None"
+            temporal_type = ""
         str_ += "\t{:26}: {}\n".format("Temporal model type", temporal_type)
 
         str_ += "\tParameters:\n"
@@ -370,220 +408,100 @@ class SkyModel(SkyModelBase):
         str_ += "\n\n"
         return str_.expandtabs(tabsize=2)
 
-
-class SkyDiffuseCube(SkyModelBase):
-    """Cube sky map template model (3D).
-
-    This is for a 3D map with an energy axis.
-    Use `~gammapy.modeling.models.TemplateSpatialModel` for 2D maps.
-
-    Parameters
-    ----------
-    map : `~gammapy.maps.Map`
-        Map template
-    norm : float
-        Norm parameter (multiplied with map values)
-    tilt : float
-        Additional tilt in the spectrum
-    reference : `~astropy.units.Quantity`
-        Reference energy of the tilt.
-    meta : dict, optional
-        Meta information, meta['filename'] will be used for serialization
-    interp_kwargs : dict
-        Interpolation keyword arguments passed to `gammapy.maps.Map.interp_by_coord`.
-        Default arguments are {'interp': 'linear', 'fill_value': 0}.
-    """
-
-    tag = "SkyDiffuseCube"
-    norm = Parameter("norm", 1)
-    tilt = Parameter("tilt", 0, unit="", frozen=True)
-    reference = Parameter("reference", "1 TeV", frozen=True)
-
-    _apply_irf_default = {"exposure": True, "psf": True, "edisp": True}
-
-    def __init__(
-        self,
-        map,
-        norm=norm.quantity,
-        tilt=tilt.quantity,
-        reference=reference.quantity,
-        meta=None,
-        interp_kwargs=None,
-        name=None,
-        filename=None,
-        apply_irf=None,
-        datasets_names=None,
-    ):
-
-        self._name = make_name(name)
-
-        self.map = map
-        self.meta = {} if meta is None else meta
-        self.filename = filename
-
-        interp_kwargs = {} if interp_kwargs is None else interp_kwargs
-        interp_kwargs.setdefault("interp", "linear")
-        interp_kwargs.setdefault("fill_value", 0)
-        self._interp_kwargs = interp_kwargs
-
-        # TODO: onve we have implement a more general and better model caching
-        #  remove this again
-        self._cached_value = None
-        self._cached_coordinates = (None, None, None)
-
-        if apply_irf is None:
-            apply_irf = self._apply_irf_default.copy()
-
-        self.apply_irf = apply_irf
-        self.datasets_names = datasets_names
-        super().__init__(norm=norm, tilt=tilt, reference=reference)
-
-    @property
-    def name(self):
-        return self._name
-
     @classmethod
-    def read(cls, filename, name=None, **kwargs):
-        """Read map from FITS file.
-
-        The default unit used if none is found in the file is ``cm-2 s-1 MeV-1 sr-1``.
+    def create(cls, spectral_model, spatial_model=None, temporal_model=None, **kwargs):
+        """Create a model instance.
 
         Parameters
         ----------
-        filename : str
-            FITS image filename.
-        name : str
-            Name of the output model
-            The default used if none is filename.
-        """
-        m = Map.read(filename, **kwargs)
-
-        if m.unit == "":
-            m.unit = "cm-2 s-1 MeV-1 sr-1"
-
-        if name is None:
-            name = Path(filename).stem
-
-        if m.geom.axes[0].name == "energy":
-            m.geom.axes[0].name = "energy_true"
-
-        return cls(m, name=name, filename=filename)
-
-    def _interpolate(self, lon, lat, energy):
-        coord = {
-            "lon": lon.to_value("deg"),
-            "lat": lat.to_value("deg"),
-            "energy_true": energy,
-        }
-        return self.map.interp_by_coord(coord, **self._interp_kwargs)
-
-    def evaluate(self, lon, lat, energy, time=None):
-        """Evaluate model.
-        passing time does not make sense here - passed just to match arguments
-        of SkyModel.evaluate"""
-        is_cached_coord = [
-            _ is coord for _, coord in zip((lon, lat, energy), self._cached_coordinates)
-        ]
-
-        # reset cache
-        if not np.all(is_cached_coord):
-            self._cached_value = None
-
-        if self._cached_value is None:
-            self._cached_coordinates = (lon, lat, energy)
-            self._cached_value = self._interpolate(lon, lat, energy)
-
-        norm = self.norm.value
-        tilt = self.tilt.value
-        reference = self.reference.quantity
-
-        tilt_factor = np.power((energy / reference).to(""), -tilt)
-
-        val = norm * self._cached_value * tilt_factor.value
-        return u.Quantity(val, self.map.unit, copy=False)
-
-    def integrate_geom(self, geom, gti=None):
-        """Integrate model on `~gammapy.maps.Geom`.
-
-        Parameters
-        ----------
-        geom : `Geom`
-            Map geometry
-        gti : `GTI`
-            GIT table (currently not being used...)
+        spectral_model : str
+            Tag to create spectral model
+        spatial_model : str
+            Tag to create spatial model
+        temporal_model : str
+            Tag to create temporal model
+        **kwargs : dict
+            Keyword arguments passed to `SkyModel`
 
         Returns
         -------
-        flux : `Map`
-            Predicted flux map
+        model : SkyModel
+            Sky model
         """
-        # TODO: implement better integration method?
-        value = self.evaluate_geom(geom)
-        value = value * geom.bin_volume()
-        return Map.from_geom(geom=geom, data=value.value, unit=value.unit)
+        spectral_model = Model.create(spectral_model, model_type="spectral")
 
-    def copy(self, name=None):
-        """A shallow copy"""
-        new = copy.copy(self)
-        new._name = make_name(name)
-        return new
+        if spatial_model:
+            spatial_model = Model.create(spatial_model, model_type="spatial")
 
-    @property
-    def position(self):
-        """`~astropy.coordinates.SkyCoord`"""
-        return self.map.geom.center_skydir
+        if temporal_model:
+            temporal_model = Model.create(temporal_model, model_type="temporal")
 
-    @property
-    def evaluation_radius(self):
-        """`~astropy.coordinates.Angle`"""
-        return np.max(self.map.geom.width) / 2.0
-
-    @property
-    def frame(self):
-        return self.position.frame.name
-
-    @classmethod
-    def from_dict(cls, data):
-        parameters = Parameters.from_dict(data["parameters"])
-
-        filename = data["filename"]
-
-        map_ = cls.read(filename).map
-
-        apply_irf = data.get("apply_irf", cls._apply_irf_default)
-        datasets_names = data.get("datasets_names")
-        name = data.get("name")
-
-        return cls.from_parameters(
-            parameters=parameters,
-            map=map_,
-            apply_irf=apply_irf,
-            datasets_names=datasets_names,
-            filename=filename,
-            name=name,
+        return cls(
+            spectral_model=spectral_model,
+            spatial_model=spatial_model,
+            temporal_model=temporal_model,
+            **kwargs,
         )
 
-    def to_dict(self):
-        data = super().to_dict()
-        data["name"] = self.name
-        data["type"] = data.pop("type")
-        data["filename"] = self.filename
 
-        # Move parameters at the end
-        data["parameters"] = data.pop("parameters")
+class FoVBackgroundModel(Model):
+    """Field of view background model
 
-        if self.apply_irf != self._apply_irf_default:
-            data["apply_irf"] = self.apply_irf
+    The background model holds the correction parameters applied to
+    the instrumental background attached to a `MapDataset` or
+    `SpectrumDataset`.
 
-        if self.datasets_names is not None:
-            data["datasets_names"] = self.datasets_names
+    Parameters
+    ----------
+    spectral_model : `~gammapy.modeling.models.SpectralModel`
+        Normalized spectral model.
+    dataset_name : str
+        Dataset name
 
-        return data
+    """
+
+    tag = ["FoVBackgroundModel", "fov-bkg"]
+
+    def __init__(self, spectral_model=None, dataset_name=None):
+        if dataset_name is None:
+            raise ValueError("Dataset name a is required argument")
+
+        self.datasets_names = [dataset_name]
+
+        if spectral_model is None:
+            spectral_model = PowerLawNormSpectralModel()
+
+        if not spectral_model.is_norm_spectral_model:
+            raise ValueError("A norm spectral model is required.")
+
+        self._spectral_model = spectral_model
+        super().__init__()
+
+    @property
+    def spectral_model(self):
+        """Spectral norm model"""
+        return self._spectral_model
+
+    @property
+    def name(self):
+        """Model name"""
+        return self.datasets_names[0] + "-bkg"
+
+    @property
+    def parameters(self):
+        """Model parameters"""
+        parameters = []
+        parameters.append(self.spectral_model.parameters)
+        return Parameters.from_stack(parameters)
 
     def __str__(self):
-        str_ = self.__class__.__name__ + "\n\n"
+        str_ = f"{self.__class__.__name__}\n\n"
+
         str_ += "\t{:26}: {}\n".format("Name", self.name)
         str_ += "\t{:26}: {}\n".format("Datasets names", self.datasets_names)
+        str_ += "\t{:26}: {}\n".format(
+            "Spectral model type", self.spectral_model.__class__.__name__
+        )
         str_ += "\tParameters:\n"
         info = _get_parameters_str(self.parameters)
         lines = info.split("\n")
@@ -592,6 +510,50 @@ class SkyDiffuseCube(SkyModelBase):
         str_ += "\n\n"
         return str_.expandtabs(tabsize=2)
 
+    def evaluate_geom(self, geom):
+        """Evaluate map"""
+        energy = geom.axes["energy"].center[:, np.newaxis, np.newaxis]
+        return self.evaluate(energy=energy)
+
+    def evaluate(self, energy):
+        """Evaluate model"""
+        return self.spectral_model(energy)
+
+    def to_dict(self, full_output=False):
+        data = {}
+        data["type"] = self.tag[0]
+        data["datasets_names"] = self.datasets_names
+        data["spectral"] = self.spectral_model.to_dict(full_output=full_output)
+        return data
+
+    @classmethod
+    def from_dict(cls, data):
+        """Create model from dict
+
+        Parameters
+        ----------
+        data : dict
+            Data dictionary
+        """
+        from gammapy.modeling.models import SPECTRAL_MODEL_REGISTRY
+
+        spectral_data = data.get("spectral")
+        if spectral_data is not None:
+            model_class = SPECTRAL_MODEL_REGISTRY.get_cls(spectral_data["type"])
+            spectral_model = model_class.from_dict(spectral_data)
+        else:
+            spectral_model = None
+
+        datasets_names = data.get("datasets_names")
+
+        if datasets_names is None:
+            raise ValueError("FoVBackgroundModel must define a dataset name")
+
+        if len(datasets_names) > 1:
+            raise ValueError("FoVBackgroundModel can only be assigned to one dataset")
+
+        return cls(spectral_model=spectral_model, dataset_name=datasets_names[0],)
+
 
 class BackgroundModel(Model):
     """Background model.
@@ -602,46 +564,41 @@ class BackgroundModel(Model):
     ----------
     map : `~gammapy.maps.Map`
         Background model map
-    norm : float
-        Background normalization
-    tilt : float
-        Additional tilt in the spectrum
-    reference : `~astropy.units.Quantity`
-        Reference energy of the tilt.
+    spectral_model : `~gammapy.modeling.models.SpectralModel`
+        Normalized spectral model,
+        default is `~gammapy.modeling.models.PowerLawNormSpectralModel`
     """
 
     tag = "BackgroundModel"
-    norm = Parameter("norm", 1, unit="", min=0)
-    tilt = Parameter("tilt", 0, unit="", frozen=True)
-    reference = Parameter("reference", "1 TeV", frozen=True)
+    map = LazyFitsData(cache=True)
 
     def __init__(
-        self,
-        map,
-        norm=norm.quantity,
-        tilt=tilt.quantity,
-        reference=reference.quantity,
-        name=None,
-        filename=None,
-        datasets_names=None,
+        self, map, spectral_model=None, name=None, filename=None, datasets_names=None,
     ):
-        axis = map.geom.get_axis_by_name("energy")
-        if axis.node_type != "edges":
-            raise ValueError('Need an integrated map, energy axis node_type="edges"')
+        if isinstance(map, Map):
+            axis = map.geom.axes["energy"]
+            if axis.node_type != "edges":
+                raise ValueError(
+                    'Need an integrated map, energy axis node_type="edges"'
+                )
 
         self.map = map
-
         self._name = make_name(name)
         self.filename = filename
 
+        if spectral_model is None:
+            spectral_model = PowerLawNormSpectralModel()
+            spectral_model.tilt.frozen = True
+
+        self.spectral_model = spectral_model
+
         if isinstance(datasets_names, list):
             if len(datasets_names) != 1:
                 raise ValueError(
                     "Currently background models can only be assigned to one dataset."
                 )
-
         self.datasets_names = datasets_names
-        super().__init__(norm=norm, tilt=tilt, reference=reference)
+        super().__init__()
 
     @property
     def name(self):
@@ -650,10 +607,27 @@ class BackgroundModel(Model):
     @property
     def energy_center(self):
         """True energy axis bin centers (`~astropy.units.Quantity`)"""
-        energy_axis = self.map.geom.get_axis_by_name("energy")
+        energy_axis = self.map.geom.axes["energy"]
         energy = energy_axis.center
         return energy[:, np.newaxis, np.newaxis]
 
+    @property
+    def spectral_model(self):
+        """`~gammapy.modeling.models.SpectralModel`"""
+        return self._spectral_model
+
+    @spectral_model.setter
+    def spectral_model(self, model):
+        if not (model is None or isinstance(model, SpectralModel)):
+            raise TypeError(f"Invalid type: {model!r}")
+        self._spectral_model = model
+
+    @property
+    def parameters(self):
+        parameters = []
+        parameters.append(self.spectral_model.parameters)
+        return Parameters.from_stack(parameters)
+
     def evaluate(self):
         """Evaluate background model.
 
@@ -662,23 +636,19 @@ class BackgroundModel(Model):
         background_map : `~gammapy.maps.Map`
             Background evaluated on the Map
         """
-        norm = self.norm.value
-        tilt = self.tilt.value
-        reference = self.reference.quantity
-        tilt_factor = np.power((self.energy_center / reference).to(""), -tilt)
-        back_values = norm * self.map.data * tilt_factor.value
+        value = self.spectral_model(self.energy_center).value
+        back_values = self.map.data * value
         return self.map.copy(data=back_values)
 
-    def to_dict(self):
+    def to_dict(self, full_output=False):
         data = {}
         data["name"] = self.name
-        data.update(super().to_dict())
+        data["type"] = self.tag
+        data["spectral"] = self.spectral_model.to_dict(full_output)
 
         if self.filename is not None:
             data["filename"] = self.filename
 
-        data["parameters"] = data.pop("parameters")
-
         if self.datasets_names is not None:
             data["datasets_names"] = self.datasets_names
 
@@ -686,6 +656,15 @@ class BackgroundModel(Model):
 
     @classmethod
     def from_dict(cls, data):
+        from gammapy.modeling.models import SPECTRAL_MODEL_REGISTRY
+
+        spectral_data = data.get("spectral")
+        if spectral_data is not None:
+            model_class = SPECTRAL_MODEL_REGISTRY.get_cls(spectral_data["type"])
+            spectral_model = model_class.from_dict(spectral_data)
+        else:
+            spectral_model = None
+
         if "filename" in data:
             bkg_map = Map.read(data["filename"])
         elif "map" in data:
@@ -699,11 +678,9 @@ class BackgroundModel(Model):
             )
             bkg_map = Map.from_geom(geom)
 
-        parameters = Parameters.from_dict(data["parameters"])
-
-        return cls.from_parameters(
-            parameters=parameters,
+        return cls(
             map=bkg_map,
+            spectral_model=spectral_model,
             name=data["name"],
             datasets_names=data.get("datasets_names"),
             filename=data.get("filename"),
@@ -738,8 +715,8 @@ class BackgroundModel(Model):
         cutout_kwargs = {"position": position, "width": width, "mode": mode}
 
         bkg_map = self.map.cutout(**cutout_kwargs)
-        parameters = self.parameters.copy()
-        return self.__class__.from_parameters(parameters, map=bkg_map, name=name)
+        spectral_model = self.spectral_model.copy()
+        return self.__class__(bkg_map, spectral_model=spectral_model, name=name)
 
     def stack(self, other, weights=None):
         """Stack background model in place.
@@ -757,8 +734,8 @@ class BackgroundModel(Model):
         self.map = bkg
 
         # reset parameter values
-        self.norm.value = 1
-        self.tilt.value = 0
+        self.spectral_model.norm.value = 1
+        self.spectral_model.tilt.value = 0
 
     def __str__(self):
         str_ = self.__class__.__name__ + "\n\n"
@@ -801,17 +778,20 @@ def create_fermi_isotropic_diffuse_model(filename, **kwargs):
     diffuse_model : `SkyModel`
         Fermi isotropic diffuse sky model.
     """
-    from .spectral import TemplateSpectralModel
-    from .spatial import ConstantSpatialModel
-
     vals = np.loadtxt(make_path(filename))
     energy = u.Quantity(vals[:, 0], "MeV", copy=False)
     values = u.Quantity(vals[:, 1], "MeV-1 s-1 cm-2", copy=False)
 
+    kwargs.setdefault("interp_kwargs", {"fill_value": None})
+
     spatial_model = ConstantSpatialModel()
-    spectral_model = TemplateSpectralModel(energy=energy, values=values, **kwargs)
+    spectral_model = (
+        TemplateSpectralModel(energy=energy, values=values, **kwargs)
+        * PowerLawNormSpectralModel()
+    )
     return SkyModel(
         spatial_model=spatial_model,
         spectral_model=spectral_model,
         name="fermi-diffuse-iso",
+        apply_irf={"psf": False, "exposure": True, "edisp": True},
     )
