@@ -1,20 +1,26 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+import operator
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose
 import astropy.units as u
 from gammapy.maps import MapAxis
 from gammapy.modeling.models import (
-    SPECTRAL_MODELS,
-    AbsorbedSpectralModel,
-    Absorption,
+    SPECTRAL_MODEL_REGISTRY,
+    BrokenPowerLawSpectralModel,
+    CompoundSpectralModel,
     ConstantSpectralModel,
+    EBLAbsorptionNormSpectralModel,
     ExpCutoffPowerLaw3FGLSpectralModel,
+    ExpCutoffPowerLawNormSpectralModel,
     ExpCutoffPowerLawSpectralModel,
     GaussianSpectralModel,
+    LogParabolaNormSpectralModel,
     LogParabolaSpectralModel,
     NaimaSpectralModel,
+    PiecewiseNormSpectralModel,
     PowerLaw2SpectralModel,
+    PowerLawNormSpectralModel,
     PowerLawSpectralModel,
     SmoothBrokenPowerLawSpectralModel,
     SuperExpCutoffPowerLaw4FGLSpectralModel,
@@ -36,7 +42,7 @@ def table_model():
     )
     dnde = model(energy)
 
-    return TemplateSpectralModel(energy, dnde, 1)
+    return TemplateSpectralModel(energy, dnde)
 
 
 TEST_MODELS = [
@@ -62,6 +68,15 @@ TEST_MODELS = [
         integral_1_10TeV=u.Quantity(3.6, "cm-2 s-1"),
         eflux_1_10TeV=u.Quantity(9.210340371976184, "TeV cm-2 s-1"),
     ),
+    dict(
+        name="norm-powerlaw",
+        model=PowerLawNormSpectralModel(
+            tilt=2 * u.Unit(""), norm=4.0 * u.Unit(""), reference=1 * u.TeV,
+        ),
+        val_at_2TeV=u.Quantity(1.0, ""),
+        integral_1_10TeV=u.Quantity(3.6, "TeV"),
+        eflux_1_10TeV=u.Quantity(9.210340371976184, "TeV2"),
+    ),
     dict(
         name="powerlaw2",
         model=PowerLaw2SpectralModel(
@@ -87,6 +102,18 @@ TEST_MODELS = [
         eflux_1_10TeV=u.Quantity(9.901735870666526, "TeV cm-2 s-1"),
         e_peak=4 * u.TeV,
     ),
+    dict(
+        name="norm-ecpl",
+        model=ExpCutoffPowerLawNormSpectralModel(
+            index=1.6 * u.Unit(""),
+            norm=4 * u.Unit(""),
+            reference=1 * u.TeV,
+            lambda_=0.1 / u.TeV,
+        ),
+        val_at_2TeV=u.Quantity(1.080321705479446, ""),
+        integral_1_10TeV=u.Quantity(3.765838739678921, "TeV"),
+        eflux_1_10TeV=u.Quantity(9.901735870666526, "TeV2"),
+    ),
     dict(
         name="ecpl_3fgl",
         model=ExpCutoffPowerLaw3FGLSpectralModel(
@@ -125,6 +152,18 @@ TEST_MODELS = [
         eflux_1_10TeV=u.Quantity(3.9586515834989267, "TeV cm-2 s-1"),
         e_peak=0.74082 * u.TeV,
     ),
+    dict(
+        name="norm-logpar",
+        model=LogParabolaNormSpectralModel(
+            alpha=2.3 * u.Unit(""),
+            norm=4 * u.Unit(""),
+            reference=1 * u.TeV,
+            beta=0.5 * u.Unit(""),
+        ),
+        val_at_2TeV=u.Quantity(0.6387956571420305, ""),
+        integral_1_10TeV=u.Quantity(2.255689748270628, "TeV"),
+        eflux_1_10TeV=u.Quantity(3.9586515834989267, "TeV2"),
+    ),
     dict(
         name="logpar10",
         model=LogParabolaSpectralModel.from_log10(
@@ -194,6 +233,18 @@ TEST_MODELS = [
         eflux_1_10TeV=u.Quantity(7.38652453, "TeV cm-2 s-1"),
         e_peak=1.7677669529663684 * u.TeV,
     ),
+    dict(
+        name="bpl",
+        model=BrokenPowerLawSpectralModel(
+            index1=1.5 * u.Unit(""),
+            index2=2.5 * u.Unit(""),
+            amplitude=4 / u.cm ** 2 / u.s / u.TeV,
+            ebreak=0.5 * u.TeV,
+        ),
+        val_at_2TeV=u.Quantity(0.125, "cm-2 s-1 TeV-1"),
+        integral_1_10TeV=u.Quantity(0.45649740094103286, "cm-2 s-1"),
+        eflux_1_10TeV=u.Quantity(0.9669999668731384, "TeV cm-2 s-1"),
+    ),
     dict(
         name="sbpl",
         model=SmoothBrokenPowerLawSpectralModel(
@@ -222,6 +273,15 @@ TEST_MODELS = [
         integral_1_10TeV=u.Quantity(13.522782989735022, "cm-2 s-1"),
         eflux_1_10TeV=u.Quantity(40.06681812966845, "TeV cm-2 s-1"),
     ),
+    dict(
+        name="pbpl",
+        model=PiecewiseNormSpectralModel(
+            energy=[1, 3, 7, 10] * u.TeV, norms=[1, 5, 3, 0.5] * u.Unit(""),
+        ),
+        val_at_2TeV=u.Quantity(2.76058404, ""),
+        integral_1_10TeV=u.Quantity(24.757573885411876, "TeV"),
+        eflux_1_10TeV=u.Quantity(117.74087966682515, "TeV2"),
+    ),
 ]
 
 # Add compound models
@@ -239,10 +299,10 @@ TEST_MODELS.append(
 TEST_MODELS.append(
     dict(
         name="compound6",
-        model=TEST_MODELS[8]["model"] + u.Quantity(4, "cm-2 s-1 TeV-1"),
-        val_at_2TeV=TEST_MODELS[8]["val_at_2TeV"] * 2,
-        integral_1_10TeV=TEST_MODELS[8]["integral_1_10TeV"] * 2,
-        eflux_1_10TeV=TEST_MODELS[8]["eflux_1_10TeV"] * 2,
+        model=TEST_MODELS[11]["model"] + u.Quantity(4, "cm-2 s-1 TeV-1"),
+        val_at_2TeV=TEST_MODELS[11]["val_at_2TeV"] * 2,
+        integral_1_10TeV=TEST_MODELS[11]["integral_1_10TeV"] * 2,
+        eflux_1_10TeV=TEST_MODELS[11]["eflux_1_10TeV"] * 2,
     )
 )
 
@@ -270,32 +330,38 @@ def test_models(spectrum):
         value = model(energy)
         assert_quantity_allclose(value, spectrum["val_at_3TeV"], rtol=1e-7)
 
-    emin = 1 * u.TeV
-    emax = 10 * u.TeV
+    energy_min = 1 * u.TeV
+    energy_max = 10 * u.TeV
     assert_quantity_allclose(
-        model.integral(emin=emin, emax=emax), spectrum["integral_1_10TeV"], rtol=1e-5
+        model.integral(energy_min=energy_min, energy_max=energy_max),
+        spectrum["integral_1_10TeV"],
+        rtol=1e-5,
     )
     assert_quantity_allclose(
-        model.energy_flux(emin=emin, emax=emax), spectrum["eflux_1_10TeV"], rtol=1e-5
+        model.energy_flux(energy_min=energy_min, energy_max=energy_max),
+        spectrum["eflux_1_10TeV"],
+        rtol=1e-5,
     )
 
     if "e_peak" in spectrum:
         assert_quantity_allclose(model.e_peak, spectrum["e_peak"], rtol=1e-2)
 
     # inverse for ConstantSpectralModel is irrelevant.
-    # inverse for Gaussian has a degeneracy
+    # inverse for Gaussian and PiecewiseNormSpectralModel have a degeneracy
     if not (
         isinstance(model, ConstantSpectralModel)
         or spectrum["name"] == "compound6"
         or spectrum["name"] == "GaussianSpectralModel"
+        or spectrum["name"] == "pbpl"
     ):
         assert_quantity_allclose(model.inverse(value), 2 * u.TeV, rtol=0.01)
 
     if "integral_infinity" in spectrum:
-        emin = 0 * u.TeV
-        emax = 10000 * u.TeV
+        energy_min = 0 * u.TeV
+        energy_max = 10000 * u.TeV
         assert_quantity_allclose(
-            model.integral(emin=emin, emax=emax), spectrum["integral_infinity"]
+            model.integral(energy_min=energy_min, energy_max=energy_max),
+            spectrum["integral_infinity"],
         )
 
     model.to_dict()
@@ -335,7 +401,7 @@ def test_to_from_dict():
     model = spectrum["model"]
 
     model_dict = model.to_dict()
-    model_class = SPECTRAL_MODELS.get_cls(model_dict["type"])
+    model_class = SPECTRAL_MODEL_REGISTRY.get_cls(model_dict["type"])
     new_model = model_class.from_dict(model_dict)
 
     assert isinstance(new_model, PowerLawSpectralModel)
@@ -345,6 +411,22 @@ def test_to_from_dict():
     assert_quantity_allclose(actual, desired)
 
 
+def test_to_from_dict_compound():
+    spectrum = TEST_MODELS[-2]
+    model = spectrum["model"]
+    assert spectrum["name"] == "compound6"
+    model_dict = model.to_dict()
+    assert model_dict["operator"] == "add"
+    model_class = SPECTRAL_MODEL_REGISTRY.get_cls(model_dict["type"])
+    new_model = model_class.from_dict(model_dict)
+
+    assert isinstance(new_model, CompoundSpectralModel)
+
+    actual = [par.value for par in new_model.parameters]
+    desired = [par.value for par in model.parameters]
+    assert_quantity_allclose(actual, desired)
+
+
 @requires_dependency("matplotlib")
 @requires_data()
 def test_table_model_from_file():
@@ -360,7 +442,9 @@ def test_table_model_from_file():
 def test_absorption():
     # absorption values for given redshift
     redshift = 0.117
-    absorption = Absorption.read_builtin("dominguez")
+    absorption = EBLAbsorptionNormSpectralModel.read_builtin(
+        "dominguez", redshift=redshift
+    )
 
     # Spectral model corresponding to PKS 2155-304 (quiescent state)
     index = 3.53
@@ -369,30 +453,26 @@ def test_absorption():
     pwl = PowerLawSpectralModel(index=index, amplitude=amplitude, reference=reference)
 
     # EBL + PWL model
-    model = AbsorbedSpectralModel(
-        spectral_model=pwl, absorption=absorption, redshift=redshift
-    )
-
+    model = pwl * absorption
     desired = u.Quantity(5.140765e-13, "TeV-1 s-1 cm-2")
     assert_quantity_allclose(model(1 * u.TeV), desired, rtol=1e-3)
-    assert model.alpha_norm.value == 1.0
+    assert model.model2.alpha_norm.value == 1.0
 
     # EBL + PWL model: test if norm of EBL=0: it mean model =pwl
-    model = AbsorbedSpectralModel(
-        spectral_model=pwl, absorption=absorption, alpha_norm=0, redshift=redshift
-    )
+    model.parameters["alpha_norm"].value = 0
     assert_quantity_allclose(model(1 * u.TeV), pwl(1 * u.TeV), rtol=1e-3)
 
     # EBL + PWL model: Test with a norm different of 1
-    model = AbsorbedSpectralModel(
-        spectral_model=pwl, absorption=absorption, alpha_norm=1.5, redshift=redshift
+    absorption = EBLAbsorptionNormSpectralModel.read_builtin(
+        "dominguez", redshift=redshift, alpha_norm=1.5
     )
+    model = pwl * absorption
     desired = u.Quantity(2.739695e-13, "TeV-1 s-1 cm-2")
-    assert model.alpha_norm.value == 1.5
+    assert model.model2.alpha_norm.value == 1.5
     assert_quantity_allclose(model(1 * u.TeV), desired, rtol=1e-3)
 
     # Test error propagation
-    model.spectral_model.amplitude.error = 0.1 * model.spectral_model.amplitude.value
+    model.model1.amplitude.error = 0.1 * model.model1.amplitude.value
     dnde, dnde_err = model.evaluate_error(1 * u.TeV)
     assert_allclose(dnde_err / dnde, 0.1)
 
@@ -400,11 +480,12 @@ def test_absorption():
 @requires_data()
 def test_absorbed_extrapolate():
     ebl_model = "dominguez"
-    z = 0.001
-    absorption = Absorption.read_builtin(ebl_model)
+    z = 0.0001
+    alpha_norm = 1
+    absorption = EBLAbsorptionNormSpectralModel.read_builtin(ebl_model)
 
-    model = absorption.table_model(z)
-    assert_allclose(model(1 * u.TeV), 1)
+    values = absorption.evaluate(1 * u.TeV, z, alpha_norm)
+    assert_allclose(values, 1)
 
 
 def test_ecpl_integrate():
@@ -433,7 +514,7 @@ def test_TemplateSpectralModel_evaluate_tiny():
     model = TemplateSpectralModel(
         energy=energy, values=values * u.Unit("MeV-1 s-1 sr-1")
     )
-    result = model.evaluate(energy, norm=1.0, tilt=0.0, reference=1 * u.TeV)
+    result = model.evaluate(energy)
     tiny = np.finfo(np.float32).tiny
     mask = abs(values) - tiny > tiny
     np.testing.assert_allclose(
@@ -443,14 +524,35 @@ def test_TemplateSpectralModel_evaluate_tiny():
     assert np.all(result[mask] == 0.0)
 
 
+def test_TemplateSpectralModel_compound():
+    energy = [1.00e06, 1.25e06, 1.58e06, 1.99e06] * u.MeV
+    values = [4.39e-7, 1.96e-7, 8.80e-7, 3.94e-7] * u.Unit("MeV-1 s-1 sr-1")
+
+    template = TemplateSpectralModel(energy=energy, values=values)
+    correction = PowerLawNormSpectralModel(norm=2)
+    model = CompoundSpectralModel(template, correction, operator=operator.mul)
+    assert np.allclose(model(energy), 2 * values)
+
+    model_mul = template * correction
+    assert isinstance(model_mul, CompoundSpectralModel)
+    assert np.allclose(model_mul(energy), 2 * values)
+
+    model_dict = model.to_dict()
+    assert model_dict["operator"] == "mul"
+    model_class = SPECTRAL_MODEL_REGISTRY.get_cls(model_dict["type"])
+    new_model = model_class.from_dict(model_dict)
+    assert isinstance(new_model, CompoundSpectralModel)
+    assert np.allclose(new_model(energy), 2 * values)
+
+
 @requires_dependency("naima")
 class TestNaimaModel:
     # Used to test model value at 2 TeV
     energy = 2 * u.TeV
 
     # Used to test model integral and energy flux
-    emin = 1 * u.TeV
-    emax = 10 * u.TeV
+    energy_min = 1 * u.TeV
+    energy_max = 10 * u.TeV
 
     # Used to that if array evaluation works
     e_array = [2, 10, 20] * u.TeV
@@ -474,10 +576,12 @@ class TestNaimaModel:
         value = model(self.energy)
         assert_quantity_allclose(value, val_at_2TeV)
         assert_quantity_allclose(
-            model.integral(emin=self.emin, emax=self.emax), integral_1_10TeV
+            model.integral(energy_min=self.energy_min, energy_max=self.energy_max),
+            integral_1_10TeV,
         )
         assert_quantity_allclose(
-            model.energy_flux(emin=self.emin, emax=self.emax), eflux_1_10TeV
+            model.energy_flux(energy_min=self.energy_min, energy_max=self.energy_max),
+            eflux_1_10TeV,
         )
         val = model(self.e_array)
         assert val.shape == self.e_array.shape
@@ -511,10 +615,14 @@ class TestNaimaModel:
         value = model(self.energy)
         assert_quantity_allclose(value, val_at_2TeV)
         assert_quantity_allclose(
-            model.integral(emin=self.emin, emax=self.emax), integral_1_10TeV, rtol=1e-5
+            model.integral(energy_min=self.energy_min, energy_max=self.energy_max),
+            integral_1_10TeV,
+            rtol=1e-5,
         )
         assert_quantity_allclose(
-            model.energy_flux(emin=self.emin, emax=self.emax), eflux_1_10TeV, rtol=1e-5
+            model.energy_flux(energy_min=self.energy_min, energy_max=self.energy_max),
+            eflux_1_10TeV,
+            rtol=1e-5,
         )
         val = model(self.e_array)
         assert val.shape == self.e_array.shape
@@ -536,10 +644,14 @@ class TestNaimaModel:
         value = model(self.energy)
         assert_quantity_allclose(value, val_at_2TeV)
         assert_quantity_allclose(
-            model.integral(emin=self.emin, emax=self.emax), integral_1_10TeV, rtol=1e-5
+            model.integral(energy_min=self.energy_min, energy_max=self.energy_max),
+            integral_1_10TeV,
+            rtol=1e-5,
         )
         assert_quantity_allclose(
-            model.energy_flux(emin=self.emin, emax=self.emax), eflux_1_10TeV, rtol=1e-5
+            model.energy_flux(energy_min=self.energy_min, energy_max=self.energy_max),
+            eflux_1_10TeV,
+            rtol=1e-5,
         )
         val = model(self.e_array)
         assert val.shape == self.e_array.shape
@@ -587,6 +699,22 @@ class TestNaimaModel:
         value = model(self.energy)
         assert_quantity_allclose(value, val_at_2TeV, rtol=1e-5)
 
+    def test_bad_init(self):
+        import naima
+
+        particle_distribution = naima.models.PowerLaw(
+            amplitude=2e33 / u.eV, e_0=10 * u.TeV, alpha=2.5
+        )
+        radiative_model = naima.radiative.PionDecay(
+            particle_distribution, nh=1 * u.cm ** -3
+        )
+        model = NaimaSpectralModel(radiative_model)
+
+        with pytest.raises(NotImplementedError):
+            NaimaSpectralModel.from_dict(model.to_dict())
+        with pytest.raises(NotImplementedError):
+            NaimaSpectralModel.from_parameters(model.parameters)
+
 
 class TestSpectralModelErrorPropagation:
     """Test spectral model error propagation.
@@ -664,3 +792,62 @@ class TestSpectralModelErrorPropagation:
 
         out = model.evaluate_error(0.1 * u.TeV)
         assert_allclose(out.data, [1.548176e-10, 1.933612e-11], rtol=1e-3)
+
+
+def test_integral_error_PowerLaw():
+    energy = np.linspace(1 * u.TeV, 10 * u.TeV, 10)
+    energy_min = energy[:-1]
+    energy_max = energy[1:]
+
+    powerlaw = PowerLawSpectralModel()
+    powerlaw.parameters["index"].error = 0.4
+    powerlaw.parameters["amplitude"].error = 1e-13
+
+    flux, flux_error = powerlaw.integral_error(energy_min, energy_max)
+
+    assert_allclose(flux.value[0] / 1e-13, 5.0, rtol=0.1)
+    assert_allclose(flux_error.value[0] / 1e-14, 8.546615432273905, rtol=0.01)
+
+
+def test_integral_error_ExpCutOffPowerLaw():
+    energy = np.linspace(1 * u.TeV, 10 * u.TeV, 10)
+    energy_min = energy[:-1]
+    energy_max = energy[1:]
+
+    exppowerlaw = ExpCutoffPowerLawSpectralModel()
+    exppowerlaw.parameters["index"].error = 0.4
+    exppowerlaw.parameters["amplitude"].error = 1e-13
+    exppowerlaw.parameters["lambda_"].error = 0.03
+
+    flux, flux_error = exppowerlaw.integral_error(energy_min, energy_max)
+
+    assert_allclose(flux.value[0] / 1e-13, 5.05855622, rtol=0.01)
+    assert_allclose(flux_error.value[0] / 1e-14, 8.90907063, rtol=0.01)
+
+
+def test_energy_flux_error_power_Law():
+    energy_min = 1 * u.TeV
+    energy_max = 10 * u.TeV
+
+    powerlaw = PowerLawSpectralModel()
+    powerlaw.parameters["index"].error = 0.4
+    powerlaw.parameters["amplitude"].error = 1e-13
+
+    enrg_flux, enrg_flux_error = powerlaw.energy_flux_error(energy_min, energy_max)
+    assert_allclose(enrg_flux.value / 1e-12, 2.303, rtol=0.001)
+    assert_allclose(enrg_flux_error.value / 1e-12, 1.347, rtol=0.001)
+
+
+def test_energy_flux_error_exp_cutoff_power_law():
+    energy_min = 1 * u.TeV
+    energy_max = 10 * u.TeV
+
+    exppowerlaw = ExpCutoffPowerLawSpectralModel()
+    exppowerlaw.parameters["index"].error = 0.4
+    exppowerlaw.parameters["amplitude"].error = 1e-13
+    exppowerlaw.parameters["lambda_"].error = 0.03
+
+    enrg_flux, enrg_flux_error = exppowerlaw.energy_flux_error(energy_min, energy_max)
+
+    assert_allclose(enrg_flux.value / 1e-12, 2.788, rtol=0.001)
+    assert_allclose(enrg_flux_error.value / 1e-12, 2.226, rtol=0.001)
