@@ -56,11 +56,17 @@ def extract_bundle(bundle, destination):
     show_default=True,
 )
 @click.option("--release", default="", help="Number of release - ex: 0.12)")
+@click.option(
+    "--all",
+    default=True,
+    is_flag=True,
+    help="Consider also other notebooks than tutorials",
+)
 @click.option("--modetutorials", default=False, hidden=True)
 @click.option("--silent", default=True, is_flag=True, hidden=True)
-def cli_download_notebooks(src, out, release, modetutorials, silent):
+def cli_download_notebooks(src, out, release, all, modetutorials, silent):
     """Download notebooks"""
-    plan = ComputePlan(src, out, release, "notebooks")
+    plan = ComputePlan(src, out, release, "notebooks", all_notebooks=all)
     if release:
         plan.getenvironment()
     down = ParallelDownload(
