@@ -19,12 +19,12 @@ values = [
 @pytest.mark.parametrize(("n_on", "mu_bkg", "result"), values)
 def test_cash_basic(n_on, mu_bkg, result):
     stat = CashCountsStatistic(n_on, mu_bkg)
-    excess = stat.excess
-    significance = stat.significance
+    excess = stat.n_sig
+    sqrt_ts = stat.sqrt_ts
     p_value = stat.p_value
 
     assert_allclose(excess, result[0])
-    assert_allclose(significance, result[1], atol=1e-5)
+    assert_allclose(sqrt_ts, result[1], atol=1e-5)
     assert_allclose(p_value, result[2], atol=1e-5)
 
 
@@ -77,7 +77,7 @@ values = [
 @pytest.mark.parametrize(("mu_bkg", "significance", "result"), values)
 def test_cash_excess_matching_significance(mu_bkg, significance, result):
     stat = CashCountsStatistic(1, mu_bkg)
-    excess = stat.excess_matching_significance(significance)
+    excess = stat.n_sig_matching_significance(significance)
 
     assert_allclose(excess, result, atol=1e-3)
 
@@ -100,13 +100,35 @@ values = [
 @pytest.mark.parametrize(("n_on", "n_off", "alpha", "result"), values)
 def test_wstat_basic(n_on, n_off, alpha, result):
     stat = WStatCountsStatistic(n_on, n_off, alpha)
-    excess = stat.excess
-    significance = stat.significance
+    excess = stat.n_sig
+    sqrt_ts = stat.sqrt_ts
     p_value = stat.p_value
 
-    assert_allclose(excess, result[0])
-    assert_allclose(significance, result[1], atol=1e-5)
-    assert_allclose(p_value, result[2], atol=1e-5)
+    assert_allclose(excess, result[0], rtol=1e-4)
+    assert_allclose(sqrt_ts, result[1], rtol=1e-4)
+    assert_allclose(p_value, result[2], rtol=1e-4)
+
+
+values = [
+    (5, 1, 1, 3, [1, 0.422261, 0.672834, 0.178305]),
+    (5, 1, 1, 1, [3.0, 1.29828, 0.19419, 1.685535]),
+    (5, 1, 1, 6, [-2, -0.75585, 0.4497382, 0.571311]),
+]
+
+
+@pytest.mark.parametrize(("n_on", "n_off", "alpha", "mu_sig", "result"), values)
+def test_wstat_with_musig(n_on, n_off, alpha, mu_sig, result):
+
+    stat = WStatCountsStatistic(n_on, n_off, alpha, mu_sig)
+    excess = stat.n_sig
+    sqrt_ts = stat.sqrt_ts
+    p_value = stat.p_value
+    del_ts = stat.ts
+
+    assert_allclose(excess, result[0], rtol=1e-4)
+    assert_allclose(sqrt_ts, result[1], rtol=1e-4)
+    assert_allclose(p_value, result[2], rtol=1e-4)
+    assert_allclose(del_ts, result[3], rtol=1e-4)
 
 
 values = [
@@ -144,18 +166,18 @@ def test_wstat_ul(n_on, n_off, alpha, result):
     stat = WStatCountsStatistic(n_on, n_off, alpha)
     ul = stat.compute_upper_limit()
 
-    assert_allclose(ul, result[0], atol=1e-5)
+    assert_allclose(ul, result[0], rtol=1e-5)
 
 
 values = [
-    ([10, 20], [0.1, 0.1], 5, [9.82966, 12.038423]),
-    ([10, 10], [0.1, 0.3], 5, [9.82966, 16.664516]),
+    ([10, 20], [0.1, 0.1], 5, [9.82966, 12.129523]),
+    ([10, 10], [0.1, 0.3], 5, [9.82966, 17.130893]),
     ([10], [0.1], 3, [4.818497]),
     (
         [[10, 20], [10, 20]],
         [[0.1, 0.1], [0.1, 0.1]],
         5,
-        [[9.82966, 12.038423], [9.82966, 12.038423]],
+        [[9.82966, 12.129523], [9.82966, 12.129523]],
     ),
 ]
 
@@ -163,6 +185,6 @@ values = [
 @pytest.mark.parametrize(("n_off", "alpha", "significance", "result"), values)
 def test_wstat_excess_matching_significance(n_off, alpha, significance, result):
     stat = WStatCountsStatistic(1, n_off, alpha)
-    excess = stat.excess_matching_significance(significance)
+    excess = stat.n_sig_matching_significance(significance)
 
-    assert_allclose(excess, result, atol=1e-3)
+    assert_allclose(excess, result, rtol=1e-2)
